# coding: utf-8

"""
    Kestra EE

    All API operations, except for Superadmin-only endpoints, require a tenant identifier in the HTTP path.<br/> Endpoints designated as Superadmin-only are not tenant-scoped.

    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501
# Custom imports
import requests
import sseclient
import json
from typing import Generator


import warnings
from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Any, Dict, List, Optional, Tuple, Union
from typing_extensions import Annotated

from pydantic import Field, StrictBool, StrictInt, StrictStr
from typing import List, Optional
from typing_extensions import Annotated
from kestrapy.models.kv_controller_api_delete_bulk_request import KVControllerApiDeleteBulkRequest
from kestrapy.models.kv_controller_api_delete_bulk_response import KVControllerApiDeleteBulkResponse
from kestrapy.models.kv_controller_typed_value import KVControllerTypedValue
from kestrapy.models.kv_entry import KVEntry
from kestrapy.models.paged_results_kv_entry import PagedResultsKVEntry
from kestrapy.models.query_filter import QueryFilter

from kestrapy.api_client import ApiClient, RequestSerialized
from kestrapy.api_response import ApiResponse
from kestrapy.rest import RESTResponseType


class KVApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client

    @validate_call
    def delete_key_value(
        self,
        namespace: Annotated[StrictStr, Field(description="The namespace id")],
        key: Annotated[StrictStr, Field(description="The key")],
        tenant: StrictStr,
        _request_timeout: Union[
        None,
        Annotated[StrictFloat, Field(gt=0)],
        Tuple[
        Annotated[StrictFloat, Field(gt=0)],
        Annotated[StrictFloat, Field(gt=0)]
        ]
        ] = None
    ) -> bool:
        """Delete a key-value pair


        :param namespace: The namespace id (required)
        :type namespace: str
                :param key: The key (required)
        :type key: str
                :param tenant: (required)
        :type tenant: str
        ,
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        """ # noqa: E501

        _param = self._delete_key_value_serialize(
            namespace=namespace,
            key=key,
            tenant=tenant,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "bool",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def delete_key_value_with_http_info(
        self,
        namespace: Annotated[StrictStr, Field(description="The namespace id")],
        key: Annotated[StrictStr, Field(description="The key")],
        tenant: StrictStr,
        _request_timeout: Union[
        None,
        Annotated[StrictFloat, Field(gt=0)],
        Tuple[
        Annotated[StrictFloat, Field(gt=0)],
        Annotated[StrictFloat, Field(gt=0)]
        ]
        ] = None
    ) -> ApiResponse[bool]:
        """Delete a key-value pair


        :param namespace: The namespace id (required)
        :type namespace: str
                :param key: The key (required)
        :type key: str
                :param tenant: (required)
        :type tenant: str
        ,
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        """ # noqa: E501

        _param = self._delete_key_value_serialize(
            namespace=namespace,
            key=key,
            tenant=tenant,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "bool",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    def _delete_key_value_serialize(
        self,
        namespace,
        key,
        tenant,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if namespace is not None:
            _path_params['namespace'] = namespace
        if key is not None:
            _path_params['key'] = key
        if tenant is not None:
            _path_params['tenant'] = tenant
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter



        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'basicAuth', 
            'bearerAuth'
        ]

        return self.api_client.param_serialize(
            method='DELETE',
            resource_path='/api/v1/{tenant}/namespaces/{namespace}/kv/{key}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats
        )





    @validate_call
    def delete_key_values(
        self,
        namespace: Annotated[StrictStr, Field(description="The namespace id")],
        tenant: StrictStr,
        kv_controller_api_delete_bulk_request: Annotated[KVControllerApiDeleteBulkRequest, Field(description="The keys")],
        _request_timeout: Union[
        None,
        Annotated[StrictFloat, Field(gt=0)],
        Tuple[
        Annotated[StrictFloat, Field(gt=0)],
        Annotated[StrictFloat, Field(gt=0)]
        ]
        ] = None
    ) -> KVControllerApiDeleteBulkResponse:
        """Bulk-delete multiple key/value pairs from the given namespace.


        :param namespace: The namespace id (required)
        :type namespace: str
                :param tenant: (required)
        :type tenant: str
                :param kv_controller_api_delete_bulk_request: The keys (required)
        :type kv_controller_api_delete_bulk_request: KVControllerApiDeleteBulkRequest
        ,
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        """ # noqa: E501

        _param = self._delete_key_values_serialize(
            namespace=namespace,
            tenant=tenant,
            kv_controller_api_delete_bulk_request=kv_controller_api_delete_bulk_request,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "KVControllerApiDeleteBulkResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def delete_key_values_with_http_info(
        self,
        namespace: Annotated[StrictStr, Field(description="The namespace id")],
        tenant: StrictStr,
        kv_controller_api_delete_bulk_request: Annotated[KVControllerApiDeleteBulkRequest, Field(description="The keys")],
        _request_timeout: Union[
        None,
        Annotated[StrictFloat, Field(gt=0)],
        Tuple[
        Annotated[StrictFloat, Field(gt=0)],
        Annotated[StrictFloat, Field(gt=0)]
        ]
        ] = None
    ) -> ApiResponse[KVControllerApiDeleteBulkResponse]:
        """Bulk-delete multiple key/value pairs from the given namespace.


        :param namespace: The namespace id (required)
        :type namespace: str
                :param tenant: (required)
        :type tenant: str
                :param kv_controller_api_delete_bulk_request: The keys (required)
        :type kv_controller_api_delete_bulk_request: KVControllerApiDeleteBulkRequest
        ,
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        """ # noqa: E501

        _param = self._delete_key_values_serialize(
            namespace=namespace,
            tenant=tenant,
            kv_controller_api_delete_bulk_request=kv_controller_api_delete_bulk_request,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "KVControllerApiDeleteBulkResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    def _delete_key_values_serialize(
        self,
        namespace,
        tenant,
        kv_controller_api_delete_bulk_request,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if namespace is not None:
            _path_params['namespace'] = namespace
        if tenant is not None:
            _path_params['tenant'] = tenant
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if kv_controller_api_delete_bulk_request is not None:
            _body_params = kv_controller_api_delete_bulk_request



        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`

        _default_content_type = (
            self.api_client.select_header_content_type(
                [
                    'application/json'
                ]
            )
        )
        if _default_content_type is not None:
            _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'basicAuth', 
            'bearerAuth'
        ]

        return self.api_client.param_serialize(
            method='DELETE',
            resource_path='/api/v1/{tenant}/namespaces/{namespace}/kv',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats
        )





    @validate_call
    def key_value(
        self,
        namespace: Annotated[StrictStr, Field(description="The namespace id")],
        key: Annotated[StrictStr, Field(description="The key")],
        tenant: StrictStr,
        _request_timeout: Union[
        None,
        Annotated[StrictFloat, Field(gt=0)],
        Tuple[
        Annotated[StrictFloat, Field(gt=0)],
        Annotated[StrictFloat, Field(gt=0)]
        ]
        ] = None
    ) -> KVControllerTypedValue:
        """Get value for a key


        :param namespace: The namespace id (required)
        :type namespace: str
                :param key: The key (required)
        :type key: str
                :param tenant: (required)
        :type tenant: str
        ,
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        """ # noqa: E501

        _param = self._key_value_serialize(
            namespace=namespace,
            key=key,
            tenant=tenant,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "KVControllerTypedValue",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def key_value_with_http_info(
        self,
        namespace: Annotated[StrictStr, Field(description="The namespace id")],
        key: Annotated[StrictStr, Field(description="The key")],
        tenant: StrictStr,
        _request_timeout: Union[
        None,
        Annotated[StrictFloat, Field(gt=0)],
        Tuple[
        Annotated[StrictFloat, Field(gt=0)],
        Annotated[StrictFloat, Field(gt=0)]
        ]
        ] = None
    ) -> ApiResponse[KVControllerTypedValue]:
        """Get value for a key


        :param namespace: The namespace id (required)
        :type namespace: str
                :param key: The key (required)
        :type key: str
                :param tenant: (required)
        :type tenant: str
        ,
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        """ # noqa: E501

        _param = self._key_value_serialize(
            namespace=namespace,
            key=key,
            tenant=tenant,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "KVControllerTypedValue",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    def _key_value_serialize(
        self,
        namespace,
        key,
        tenant,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if namespace is not None:
            _path_params['namespace'] = namespace
        if key is not None:
            _path_params['key'] = key
        if tenant is not None:
            _path_params['tenant'] = tenant
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter



        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'basicAuth', 
            'bearerAuth'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/api/v1/{tenant}/namespaces/{namespace}/kv/{key}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats
        )





    @validate_call
    def list_all_keys(
        self,
        page: Annotated[StrictInt, Field(description="The current page")],
        size: Annotated[StrictInt, Field(description="The current page size")],
        tenant: StrictStr,
        sort: Annotated[Optional[List[StrictStr]], Field(description="The sort of current page")] = None,
        filters: Annotated[Optional[List[QueryFilter]], Field(description="Filters")] = None,
        _request_timeout: Union[
        None,
        Annotated[StrictFloat, Field(gt=0)],
        Tuple[
        Annotated[StrictFloat, Field(gt=0)],
        Annotated[StrictFloat, Field(gt=0)]
        ]
        ] = None
    ) -> PagedResultsKVEntry:
        """List all keys


        :param page: The current page (required)
        :type page: int
                :param size: The current page size (required)
        :type size: int
                :param tenant: (required)
        :type tenant: str
                :param sort: The sort of current page
        :type sort: List[str]
                :param filters: Filters
        :type filters: List[QueryFilter]
        ,
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        """ # noqa: E501

        _param = self._list_all_keys_serialize(
            page=page,
            size=size,
            tenant=tenant,
            sort=sort,
            filters=filters,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "PagedResultsKVEntry",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def list_all_keys_with_http_info(
        self,
        page: Annotated[StrictInt, Field(description="The current page")],
        size: Annotated[StrictInt, Field(description="The current page size")],
        tenant: StrictStr,
        sort: Annotated[Optional[List[StrictStr]], Field(description="The sort of current page")] = None,
        filters: Annotated[Optional[List[QueryFilter]], Field(description="Filters")] = None,
        _request_timeout: Union[
        None,
        Annotated[StrictFloat, Field(gt=0)],
        Tuple[
        Annotated[StrictFloat, Field(gt=0)],
        Annotated[StrictFloat, Field(gt=0)]
        ]
        ] = None
    ) -> ApiResponse[PagedResultsKVEntry]:
        """List all keys


        :param page: The current page (required)
        :type page: int
                :param size: The current page size (required)
        :type size: int
                :param tenant: (required)
        :type tenant: str
                :param sort: The sort of current page
        :type sort: List[str]
                :param filters: Filters
        :type filters: List[QueryFilter]
        ,
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        """ # noqa: E501

        _param = self._list_all_keys_serialize(
            page=page,
            size=size,
            tenant=tenant,
            sort=sort,
            filters=filters,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "PagedResultsKVEntry",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    def _list_all_keys_serialize(
        self,
        page,
        size,
        tenant,
        sort,
        filters,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
            'sort': 'csv',
            'filters': 'csv',
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if tenant is not None:
            _path_params['tenant'] = tenant
        # process the query parameters
        if page is not None:
            
            _query_params.append(('page', page))
            
        if size is not None:
            
            _query_params.append(('size', size))
            
        if sort is not None:
            
            _query_params.append(('sort', sort))
            
        if filters is not None:
            
            _query_params.append(('filters', filters))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter



        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'basicAuth', 
            'bearerAuth'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/api/v1/{tenant}/kv',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats
        )





    @validate_call
    def list_keys(
        self,
        namespace: Annotated[StrictStr, Field(description="The namespace id")],
        tenant: StrictStr,
        _request_timeout: Union[
        None,
        Annotated[StrictFloat, Field(gt=0)],
        Tuple[
        Annotated[StrictFloat, Field(gt=0)],
        Annotated[StrictFloat, Field(gt=0)]
        ]
        ] = None
    ) -> List[KVEntry]:
        """(Deprecated) List all keys for a namespace


        :param namespace: The namespace id (required)
        :type namespace: str
                :param tenant: (required)
        :type tenant: str
        ,
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        """ # noqa: E501
        warnings.warn("GET /api/v1/{tenant}/namespaces/{namespace}/kv is deprecated.", DeprecationWarning)

        _param = self._list_keys_serialize(
            namespace=namespace,
            tenant=tenant,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[KVEntry]",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def list_keys_with_http_info(
        self,
        namespace: Annotated[StrictStr, Field(description="The namespace id")],
        tenant: StrictStr,
        _request_timeout: Union[
        None,
        Annotated[StrictFloat, Field(gt=0)],
        Tuple[
        Annotated[StrictFloat, Field(gt=0)],
        Annotated[StrictFloat, Field(gt=0)]
        ]
        ] = None
    ) -> ApiResponse[List[KVEntry]]:
        """(Deprecated) List all keys for a namespace


        :param namespace: The namespace id (required)
        :type namespace: str
                :param tenant: (required)
        :type tenant: str
        ,
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        """ # noqa: E501
        warnings.warn("GET /api/v1/{tenant}/namespaces/{namespace}/kv is deprecated.", DeprecationWarning)

        _param = self._list_keys_serialize(
            namespace=namespace,
            tenant=tenant,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[KVEntry]",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    def _list_keys_serialize(
        self,
        namespace,
        tenant,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if namespace is not None:
            _path_params['namespace'] = namespace
        if tenant is not None:
            _path_params['tenant'] = tenant
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter



        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'basicAuth', 
            'bearerAuth'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/api/v1/{tenant}/namespaces/{namespace}/kv',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats
        )





    @validate_call
    def list_keys_with_inheritence(
        self,
        namespace: Annotated[StrictStr, Field(description="The namespace id")],
        tenant: StrictStr,
        _request_timeout: Union[
        None,
        Annotated[StrictFloat, Field(gt=0)],
        Tuple[
        Annotated[StrictFloat, Field(gt=0)],
        Annotated[StrictFloat, Field(gt=0)]
        ]
        ] = None
    ) -> List[KVEntry]:
        """List all keys for inherited namespaces


        :param namespace: The namespace id (required)
        :type namespace: str
                :param tenant: (required)
        :type tenant: str
        ,
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        """ # noqa: E501

        _param = self._list_keys_with_inheritence_serialize(
            namespace=namespace,
            tenant=tenant,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[KVEntry]",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def list_keys_with_inheritence_with_http_info(
        self,
        namespace: Annotated[StrictStr, Field(description="The namespace id")],
        tenant: StrictStr,
        _request_timeout: Union[
        None,
        Annotated[StrictFloat, Field(gt=0)],
        Tuple[
        Annotated[StrictFloat, Field(gt=0)],
        Annotated[StrictFloat, Field(gt=0)]
        ]
        ] = None
    ) -> ApiResponse[List[KVEntry]]:
        """List all keys for inherited namespaces


        :param namespace: The namespace id (required)
        :type namespace: str
                :param tenant: (required)
        :type tenant: str
        ,
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        """ # noqa: E501

        _param = self._list_keys_with_inheritence_serialize(
            namespace=namespace,
            tenant=tenant,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[KVEntry]",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    def _list_keys_with_inheritence_serialize(
        self,
        namespace,
        tenant,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if namespace is not None:
            _path_params['namespace'] = namespace
        if tenant is not None:
            _path_params['tenant'] = tenant
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter



        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'basicAuth', 
            'bearerAuth'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/api/v1/{tenant}/namespaces/{namespace}/kv/inheritance',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats
        )





    @validate_call
    def set_key_value(
        self,
        namespace: Annotated[StrictStr, Field(description="The namespace id")],
        key: Annotated[StrictStr, Field(description="The key")],
        tenant: StrictStr,
        body: Annotated[StrictStr, Field(description="The value of the key")],
        _request_timeout: Union[
        None,
        Annotated[StrictFloat, Field(gt=0)],
        Tuple[
        Annotated[StrictFloat, Field(gt=0)],
        Annotated[StrictFloat, Field(gt=0)]
        ]
        ] = None
    ) -> None:
        """Puts a key-value pair in store


        :param namespace: The namespace id (required)
        :type namespace: str
                :param key: The key (required)
        :type key: str
                :param tenant: (required)
        :type tenant: str
                :param body: The value of the key (required)
        :type body: str
        ,
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        """ # noqa: E501

        _param = self._set_key_value_serialize(
            namespace=namespace,
            key=key,
            tenant=tenant,
            body=body,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def set_key_value_with_http_info(
        self,
        namespace: Annotated[StrictStr, Field(description="The namespace id")],
        key: Annotated[StrictStr, Field(description="The key")],
        tenant: StrictStr,
        body: Annotated[StrictStr, Field(description="The value of the key")],
        _request_timeout: Union[
        None,
        Annotated[StrictFloat, Field(gt=0)],
        Tuple[
        Annotated[StrictFloat, Field(gt=0)],
        Annotated[StrictFloat, Field(gt=0)]
        ]
        ] = None
    ) -> ApiResponse[None]:
        """Puts a key-value pair in store


        :param namespace: The namespace id (required)
        :type namespace: str
                :param key: The key (required)
        :type key: str
                :param tenant: (required)
        :type tenant: str
                :param body: The value of the key (required)
        :type body: str
        ,
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        """ # noqa: E501

        _param = self._set_key_value_serialize(
            namespace=namespace,
            key=key,
            tenant=tenant,
            body=body,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    def _set_key_value_serialize(
        self,
        namespace,
        key,
        tenant,
        body,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if namespace is not None:
            _path_params['namespace'] = namespace
        if key is not None:
            _path_params['key'] = key
        if tenant is not None:
            _path_params['tenant'] = tenant
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if body is not None:
            _body_params = body




        # set the HTTP header `Content-Type`

        _default_content_type = (
            self.api_client.select_header_content_type(
                [
                    'text/plain'
                ]
            )
        )
        if _default_content_type is not None:
            _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'basicAuth', 
            'bearerAuth'
        ]

        return self.api_client.param_serialize(
            method='PUT',
            resource_path='/api/v1/{tenant}/namespaces/{namespace}/kv/{key}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats
        )







# coding: utf-8

"""
    Kestra EE

    All API operations, except for Superadmin-only endpoints, require a tenant identifier in the HTTP path.<br/> Endpoints designated as Superadmin-only are not tenant-scoped.

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest
from kestrapy import Configuration, KestraClient, IAMGroupControllerApiCreateGroupRequest, ApiIds, \
    IAMUserControllerApiCreateOrUpdateUserRequest, ApiAutocomplete, IAMGroupControllerApiUpdateGroupRequest, \
    GroupIdentifierMembership

class TestGroupsApi(unittest.TestCase):
    """GroupsApi unit test stubs"""

    def setUp(self) -> None:
        configuration = Configuration()
        configuration.host = "http://localhost:9902"
        configuration.username = "root@root.com"
        configuration.password = "Root!1234"

        self.kestra_client = KestraClient(configuration)
        self.tenant = "main"


    def tearDown(self) -> None:
        pass

    def test_add_user_to_group(self) -> None:
        """Test case for add_user_to_group

        Add a user to a group
        """
        group = IAMGroupControllerApiCreateGroupRequest(
            name="test_add_user_to_group",
            description="An example group"
        )

        created_group = self.kestra_client.groups.create_group(
            tenant=self.tenant,
            iam_group_controller_api_create_group_request=group
        )

        user = IAMUserControllerApiCreateOrUpdateUserRequest(
            email="test_add_user_to_group@kestra.io"
        )

        created_user = self.kestra_client.users.create_user(
            iam_user_controller_api_create_or_update_user_request=user
        )

        member = self.kestra_client.groups.add_user_to_group(
            tenant=self.tenant,
            id=created_group.id,
            user_id=created_user.id
        )

        assert any(getattr(g, 'id', None) == created_group.id for g in member.groups)

    def test_autocomplete_groups(self) -> None:
        """Test case for autocomplete_groups

        List groups for autocomplete
        """
        prefix="test_auto"
        group = IAMGroupControllerApiCreateGroupRequest(
            name=prefix + "complete_groups",
            description="An example group"
        )

        created = self.kestra_client.groups.create_group(
            tenant=self.tenant,
            iam_group_controller_api_create_group_request=group
        )

        apiAutocomplete = ApiAutocomplete(q=prefix)

        results = self.kestra_client.groups.autocomplete_groups(
            tenant=self.tenant,
            api_autocomplete=apiAutocomplete
        )

        assert any(getattr(r, 'id', None) == created.id or getattr(r, 'name', None) == created.name for r in results)

    def test_create_group(self) -> None:
        """Test case for create_group

        Create a group
        """
        group = IAMGroupControllerApiCreateGroupRequest(
            name="test_create_group",
            description="An example group"
        )

        created = self.kestra_client.groups.create_group(
            tenant=self.tenant,
            iam_group_controller_api_create_group_request=group
        )
        assert created.name == group.name

    def test_delete_group(self) -> None:
        group = IAMGroupControllerApiCreateGroupRequest(
            name="test_delete_group",
            description="An example group"
        )

        created = self.kestra_client.groups.create_group(
            tenant=self.tenant,
            iam_group_controller_api_create_group_request=group
        )

        self.kestra_client.groups.delete_group(
            tenant=self.tenant,
            id=created.id
        )

    def test_delete_user_from_group(self) -> None:
        """Test case for delete_user_from_group

        Remove a user from a group
        """
        group = IAMGroupControllerApiCreateGroupRequest(
            name="test_delete_user_from_group",
            description="An example group"
        )

        created_group = self.kestra_client.groups.create_group(
            tenant=self.tenant,
            iam_group_controller_api_create_group_request=group
        )

        user = IAMUserControllerApiCreateOrUpdateUserRequest(
            email="test_delete_user_from_group@kestra.io"
        )

        created_user = self.kestra_client.users.create_user(
            iam_user_controller_api_create_or_update_user_request=user
        )

        self.kestra_client.groups.add_user_to_group(
            tenant=self.tenant,
            id=created_group.id,
            user_id=created_user.id
        )

        iAMGroupControllerApiGroupMember = self.kestra_client.groups.delete_user_from_group(
            tenant=self.tenant,
            id=created_group.id,
            user_id=created_user.id
        )
        assert iAMGroupControllerApiGroupMember.groups.__contains__(created_group.id) is False

    def test_get_group(self) -> None:
        """Test case for get_group

        Retrieve a group
        """
        group = IAMGroupControllerApiCreateGroupRequest(
            name="test_get_group",
            description="An example group"
        )

        created = self.kestra_client.groups.create_group(
            tenant=self.tenant,
            iam_group_controller_api_create_group_request=group
        )

        fetched = self.kestra_client.groups.group(
            tenant=self.tenant,
            id=created.id
        )

        assert fetched.id == created.id

    def test_list_group_ids(self) -> None:
        """Test case for list_group_ids

        List groups by ids
        """
        group = IAMGroupControllerApiCreateGroupRequest(
            name="test_list_group_ids",
            description="An example group"
        )

        created = self.kestra_client.groups.create_group(
            tenant=self.tenant,
            iam_group_controller_api_create_group_request=group
        )

        apiIds = ApiIds(ids=[created.id])

        fetched = self.kestra_client.groups.list_group_ids(
            tenant=self.tenant,
            api_ids=apiIds
        )

        assert len(fetched) > 0

    def test_search_group_members(self) -> None:
        """Test case for search_group_members

        Search for users in a group
        """
        group = IAMGroupControllerApiCreateGroupRequest(
            name="test_search_group_members",
            description="An example group"
        )

        created_group = self.kestra_client.groups.create_group(
            tenant=self.tenant,
            iam_group_controller_api_create_group_request=group
        )

        user = IAMUserControllerApiCreateOrUpdateUserRequest(
            email="test_search_group_members@kestra.io"
        )

        created_user = self.kestra_client.users.create_user(
            iam_user_controller_api_create_or_update_user_request=user
        )

        self.kestra_client.groups.add_user_to_group(
            tenant=self.tenant,
            id=created_group.id,
            user_id=created_user.id
        )

        members = self.kestra_client.groups.search_group_members(
            tenant=self.tenant,
            id=created_group.id,
            q=created_user.email,
            page=1,
            size=10
        )

        assert any(getattr(m, 'id', None) == created_user.id for m in members.results)

    def test_search_groups(self) -> None:
        """Test case for search_groups

        Search for groups
        """
        name = "test_search_groups"
        group = IAMGroupControllerApiCreateGroupRequest(
            name=name,
            description="An example group"
        )

        created = self.kestra_client.groups.create_group(
            tenant=self.tenant,
            iam_group_controller_api_create_group_request=group
        )

        results = self.kestra_client.groups.search_groups(
            tenant=self.tenant,
            page=1,
            size=10,
            q=name
        )

        assert any(getattr(r, 'id', None) == created.id for r in results.results)

    def test_set_user_membership_for_group(self) -> None:
        """Test case for set_user_membership_for_group

        Update a user's membership type in a group
        """
        group = IAMGroupControllerApiCreateGroupRequest(
            name="test_set_user_membership_for_group",
            description="An example group"
        )

        created_group = self.kestra_client.groups.create_group(
            tenant=self.tenant,
            iam_group_controller_api_create_group_request=group
        )

        user = IAMUserControllerApiCreateOrUpdateUserRequest(
            email="test_set_user_membership_for_group@kestra.io"
        )

        created_user = self.kestra_client.users.create_user(
            iam_user_controller_api_create_or_update_user_request=user
        )

        self.kestra_client.groups.add_user_to_group(
            tenant=self.tenant,
            id=created_group.id,
            user_id=created_user.id
        )

        member = self.kestra_client.groups.set_user_membership_for_group(
            tenant=self.tenant,
            id=created_group.id,
            user_id=created_user.id,
            membership=GroupIdentifierMembership.OWNER
        )

        assert any(getattr(g, 'id', None) == created_group.id for g in member.groups)

    def test_update_group(self) -> None:
        """Test case for update_group

        Update a group
        """
        group = IAMGroupControllerApiCreateGroupRequest(
            name="test_update_group",
            description="An example group"
        )

        created = self.kestra_client.groups.create_group(
            tenant=self.tenant,
            iam_group_controller_api_create_group_request=group
        )

        update_desc = "Updated description"

        updatedRequest = IAMGroupControllerApiUpdateGroupRequest(
            name=created.name,
            description = update_desc
        )

        updated = self.kestra_client.groups.update_group(
            tenant=self.tenant,
            id=created.id,
            iam_group_controller_api_update_group_request=updatedRequest
        )

        assert getattr(updated, 'description', None) == update_desc


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Kestra EE

    All API operations, except for Superadmin-only endpoints, require a tenant identifier in the HTTP path.<br/> Endpoints designated as Superadmin-only are not tenant-scoped.

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest
from kestrapy import (
    Configuration,
    KestraClient,
    Namespace,
    ApiAutocomplete,
    ApiSecretValue, ApiSecretMetaEE,
)


class TestNamespacesApi(unittest.TestCase):
    """NamespacesApi unit test stubs"""

    def setUp(self) -> None:
        configuration = Configuration()
        configuration.host = "http://localhost:9902"
        configuration.username = "root@root.com"
        configuration.password = "Root!1234"

        self.kestra_client = KestraClient(configuration)
        self.tenant = "main"

    def tearDown(self) -> None:
        pass

    def test_autocomplete_namespaces(self) -> None:
        """Test case for autocomplete_namespaces

        List namespaces for autocomplete
        """
        prefix = "test_autocomplete_namespaces"
        ns = Namespace(id=prefix, deleted=False)

        created = self.kestra_client.namespaces.create_namespace(tenant=self.tenant, namespace=ns)

        api_autocomplete = ApiAutocomplete(q=prefix)
        results = self.kestra_client.namespaces.autocomplete_namespaces(tenant=self.tenant, api_autocomplete=api_autocomplete)

        assert any(r == created.id for r in results)

    def test_create_namespace(self) -> None:
        """Test case for create_namespace

        Create a namespace
        """
        ns_id = "test_create_namespace"
        ns = Namespace(id=ns_id, deleted=False)

        created = self.kestra_client.namespaces.create_namespace(tenant=self.tenant, namespace=ns)
        assert getattr(created, 'id', None) == ns_id

    def test_delete_namespace(self) -> None:
        """Test case for delete_namespace

        Delete a namespace
        """
        ns_id = "test_delete_namespace"
        ns = Namespace(id=ns_id, deleted=False)

        created = self.kestra_client.namespaces.create_namespace(tenant=self.tenant, namespace=ns)

        self.kestra_client.namespaces.delete_namespace(id=created.id, tenant=self.tenant)

        # fetching should fail
        with self.assertRaises(Exception):
            self.kestra_client.namespaces.namespace(id=created.id, tenant=self.tenant)

    def test_get_inherited_secrets(self) -> None:
        """Test case for get_inherited_secrets

        List inherited secrets
        """
        ns_id = "test_get_inherited_secrets"
        ns = Namespace(id=ns_id, deleted=False)
        created = self.kestra_client.namespaces.create_namespace(tenant=self.tenant, namespace=ns)

        inherited = self.kestra_client.namespaces.inherited_secrets(namespace=created.id, tenant=self.tenant)
        assert isinstance(inherited, dict)

    def test_get_namespace(self) -> None:
        """Test case for get_namespace

        Get a namespace
        """
        ns_id = "test_get_namespace"
        ns = Namespace(id=ns_id, deleted=False)
        created = self.kestra_client.namespaces.create_namespace(tenant=self.tenant, namespace=ns)

        fetched = self.kestra_client.namespaces.namespace(id=created.id, tenant=self.tenant)
        assert getattr(fetched, 'id', None) == created.id

    def test_inherited_plugin_defaults(self) -> None:
        """Test case for inherited_plugin_defaults

        List inherited plugin defaults
        """
        ns_id = "test_inherited_plugin_defaults"
        ns = Namespace(id=ns_id, deleted=False)
        created = self.kestra_client.namespaces.create_namespace(tenant=self.tenant, namespace=ns)

        defaults = self.kestra_client.namespaces.inherited_plugin_defaults(id=created.id, tenant=self.tenant)
        assert isinstance(defaults, list)

    def test_inherited_variables(self) -> None:
        """Test case for inherited_variables

        List inherited variables
        """
        ns_id = "test_inherited_variables"
        ns = Namespace(id=ns_id, deleted=False)
        created = self.kestra_client.namespaces.create_namespace(tenant=self.tenant, namespace=ns)

        variables = self.kestra_client.namespaces.inherited_variables(id=created.id, tenant=self.tenant)
        assert isinstance(variables, dict)

    def test_list_namespace_secrets(self) -> None:
        """Test case for list_namespace_secrets

        Get secrets for a namespace
        """
        ns_id = "test_list_namespace_secrets"
        ns = Namespace(id=ns_id, deleted=False)
        created = self.kestra_client.namespaces.create_namespace(tenant=self.tenant, namespace=ns)

        # list secrets with empty filters
        entries = self.kestra_client.namespaces.list_namespace_secrets(namespace=created.id, page=1, size=10, filters=[], tenant=self.tenant)
        assert getattr(entries, 'results', None) is not None

    def test_patch_secret(self) -> None:
        """Test case for patch_secret

        Patch a secret metadata for a namespace
        """
        ns_id = "test_patch_secret"
        ns = Namespace(id=ns_id, deleted=False)
        created = self.kestra_client.namespaces.create_namespace(tenant=self.tenant, namespace=ns)

        api_secret_meta = ApiSecretMetaEE(
            key="test_patch_secret_key",
            description="patched description",
            tags=[],
        )
        metas = self.kestra_client.namespaces.patch_secret(
            namespace=created.id,
            key=api_secret_meta.key,
            tenant=self.tenant,
            api_secret_meta_ee=api_secret_meta,
        )
        assert isinstance(metas, list)

    def test_put_secrets(self) -> None:
        """Test case for put_secrets

        Update secrets for a namespace
        """
        ns_id = "test_put_secrets4"
        ns = Namespace(id=ns_id, deleted=False)
        created = self.kestra_client.namespaces.create_namespace(tenant=self.tenant, namespace=ns)

        secret = ApiSecretValue(
            key="test-key",
            value="super-secret-value",
            description="Test secret for put_secrets"
        )

        response = self.kestra_client.namespaces.put_secrets(
            namespace=ns_id,
            tenant=self.tenant,
            api_secret_value=secret
        )

        assert isinstance(response, list)

    def test_search_namespaces(self) -> None:
        """Test case for search_namespaces

        Search for namespaces
        """
        ns_id = "test_search_namespaces"
        ns = Namespace(id=ns_id, deleted=False)
        created = self.kestra_client.namespaces.create_namespace(tenant=self.tenant, namespace=ns)

        results = self.kestra_client.namespaces.search_namespaces(page=1, size=10, existing=False, tenant=self.tenant, q=ns_id)
        assert any(getattr(r, 'id', None) == created.id for r in results.results)

    def test_update_namespace(self) -> None:
        """Test case for update_namespace

        Update a namespace
        """
        ns_id = "test_update_namespace"
        ns = Namespace(id=ns_id, deleted=False)
        created = self.kestra_client.namespaces.create_namespace(tenant=self.tenant, namespace=ns)

        fetched = self.kestra_client.namespaces.namespace(id=created.id, tenant=self.tenant)
        assert getattr(fetched, 'id', None) == created.id



if __name__ == '__main__':
    unittest.main()



# coding: utf-8

# flake8: noqa
"""
    Kestra EE

    All API operations, except for Superadmin-only endpoints, require a tenant identifier in the HTTP path.<br/> Endpoints designated as Superadmin-only are not tenant-scoped.

    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501

# import models into model package
from kestrapy.models.abstract_flow import AbstractFlow
from kestrapy.models.abstract_graph import AbstractGraph
from kestrapy.models.abstract_graph_branch_type import AbstractGraphBranchType
from kestrapy.models.abstract_metric_entry_object import AbstractMetricEntryObject
from kestrapy.models.abstract_trigger import AbstractTrigger
from kestrapy.models.abstract_trigger_for_execution import AbstractTriggerForExecution
from kestrapy.models.abstract_user import AbstractUser
from kestrapy.models.abstract_user_tenant_identity_provider import AbstractUserTenantIdentityProvider
from kestrapy.models.action import Action
from kestrapy.models.api_auth import ApiAuth
from kestrapy.models.api_autocomplete import ApiAutocomplete
from kestrapy.models.api_group_summary import ApiGroupSummary
from kestrapy.models.api_ids import ApiIds
from kestrapy.models.api_patch_super_admin_request import ApiPatchSuperAdminRequest
from kestrapy.models.api_role_summary import ApiRoleSummary
from kestrapy.models.api_secret_list_response_api_secret_meta import ApiSecretListResponseApiSecretMeta
from kestrapy.models.api_secret_meta import ApiSecretMeta
from kestrapy.models.api_secret_meta_ee import ApiSecretMetaEE
from kestrapy.models.api_secret_tag import ApiSecretTag
from kestrapy.models.api_secret_value import ApiSecretValue
from kestrapy.models.api_tenant import ApiTenant
from kestrapy.models.api_tenant_summary import ApiTenantSummary
from kestrapy.models.api_token import ApiToken
from kestrapy.models.api_token_list import ApiTokenList
from kestrapy.models.api_user import ApiUser
from kestrapy.models.app_response import AppResponse
from kestrapy.models.app_response_ui_layout import AppResponseUILayout
from kestrapy.models.apps_controller_api_app import AppsControllerApiApp
from kestrapy.models.apps_controller_api_app_catalog_item import AppsControllerApiAppCatalogItem
from kestrapy.models.apps_controller_api_app_source import AppsControllerApiAppSource
from kestrapy.models.apps_controller_api_app_tags import AppsControllerApiAppTags
from kestrapy.models.apps_controller_api_bulk_import_response import AppsControllerApiBulkImportResponse
from kestrapy.models.apps_controller_api_bulk_import_response_error import AppsControllerApiBulkImportResponseError
from kestrapy.models.apps_controller_api_bulk_operation_request import AppsControllerApiBulkOperationRequest
from kestrapy.models.assertion import Assertion
from kestrapy.models.assertion_result import AssertionResult
from kestrapy.models.assertion_run_error import AssertionRunError
from kestrapy.models.asset import Asset
from kestrapy.models.asset_identifier import AssetIdentifier
from kestrapy.models.asset_topology_graph import AssetTopologyGraph
from kestrapy.models.asset_topology_graph_edge import AssetTopologyGraphEdge
from kestrapy.models.asset_topology_graph_node import AssetTopologyGraphNode
from kestrapy.models.asset_topology_graph_node_node_type import AssetTopologyGraphNodeNodeType
from kestrapy.models.assets_controller_api_asset import AssetsControllerApiAsset
from kestrapy.models.assets_controller_api_asset_usage import AssetsControllerApiAssetUsage
from kestrapy.models.assets_declaration import AssetsDeclaration
from kestrapy.models.assets_in_out import AssetsInOut
from kestrapy.models.attribute_reference import AttributeReference
from kestrapy.models.audit_log import AuditLog
from kestrapy.models.audit_log_controller_api_audit_log_item import AuditLogControllerApiAuditLogItem
from kestrapy.models.audit_log_controller_audit_log_diff import AuditLogControllerAuditLogDiff
from kestrapy.models.audit_log_controller_audit_log_option import AuditLogControllerAuditLogOption
from kestrapy.models.audit_log_controller_find_request import AuditLogControllerFindRequest
from kestrapy.models.audit_log_detail import AuditLogDetail
from kestrapy.models.auth_controller_auth import AuthControllerAuth
from kestrapy.models.auth_controller_invitation_user_request import AuthControllerInvitationUserRequest
from kestrapy.models.auth_controller_reset_password_request import AuthControllerResetPasswordRequest
from kestrapy.models.backfill import Backfill
from kestrapy.models.banner import Banner
from kestrapy.models.banner_type import BannerType
from kestrapy.models.base_audit_log import BaseAuditLog
from kestrapy.models.base_resource_patch_request import BaseResourcePatchRequest
from kestrapy.models.base_resource_scim_resource import BaseResourceScimResource
from kestrapy.models.base_resource_search_request import BaseResourceSearchRequest
from kestrapy.models.basic_auth_credentials import BasicAuthCredentials
from kestrapy.models.binding import Binding
from kestrapy.models.binding_type import BindingType
from kestrapy.models.blueprint import Blueprint
from kestrapy.models.blueprint_controller_api_blueprint_item import BlueprintControllerApiBlueprintItem
from kestrapy.models.blueprint_controller_api_blueprint_item_with_source import BlueprintControllerApiBlueprintItemWithSource
from kestrapy.models.blueprint_controller_api_blueprint_tag_item import BlueprintControllerApiBlueprintTagItem
from kestrapy.models.blueprint_controller_api_flow_blueprint import BlueprintControllerApiFlowBlueprint
from kestrapy.models.blueprint_controller_flow_blueprint_create_or_update import BlueprintControllerFlowBlueprintCreateOrUpdate
from kestrapy.models.blueprint_controller_kind import BlueprintControllerKind
from kestrapy.models.blueprint_controller_use_blueprint_template_request import BlueprintControllerUseBlueprintTemplateRequest
from kestrapy.models.blueprint_controller_use_blueprint_template_response import BlueprintControllerUseBlueprintTemplateResponse
from kestrapy.models.blueprint_template import BlueprintTemplate
from kestrapy.models.blueprint_with_flow_entity import BlueprintWithFlowEntity
from kestrapy.models.breakpoint import Breakpoint
from kestrapy.models.bulk_error_response import BulkErrorResponse
from kestrapy.models.bulk_import_apps_request import BulkImportAppsRequest
from kestrapy.models.bulk_response import BulkResponse
from kestrapy.models.cache import Cache
from kestrapy.models.chart_chart_option import ChartChartOption
from kestrapy.models.chart_filters_overrides import ChartFiltersOverrides
from kestrapy.models.check import Check
from kestrapy.models.check_behavior import CheckBehavior
from kestrapy.models.check_style import CheckStyle
from kestrapy.models.concurrency import Concurrency
from kestrapy.models.concurrency_behavior import ConcurrencyBehavior
from kestrapy.models.concurrency_limit import ConcurrencyLimit
from kestrapy.models.condition import Condition
from kestrapy.models.conversion_service_provider import ConversionServiceProvider
from kestrapy.models.convertible_multi_values_string import ConvertibleMultiValuesString
from kestrapy.models.convertible_values_list_string import ConvertibleValuesListString
from kestrapy.models.create_api_token_request import CreateApiTokenRequest
from kestrapy.models.create_api_token_response import CreateApiTokenResponse
from kestrapy.models.create_namespace_file_request import CreateNamespaceFileRequest
from kestrapy.models.create_security_integration_request import CreateSecurityIntegrationRequest
from kestrapy.models.crud_event_type import CrudEventType
from kestrapy.models.daily_execution_statistics import DailyExecutionStatistics
from kestrapy.models.daily_execution_statistics_duration import DailyExecutionStatisticsDuration
from kestrapy.models.daily_execution_statistics_execution_counts import DailyExecutionStatisticsExecutionCounts
from kestrapy.models.dashboard import Dashboard
from kestrapy.models.dashboard_controller_preview_request import DashboardControllerPreviewRequest
from kestrapy.models.delete_triggers_by_query_request import DeleteTriggersByQueryRequest
from kestrapy.models.deleted_interface import DeletedInterface
from kestrapy.models.depends_on import DependsOn
from kestrapy.models.documentation_with_schema import DocumentationWithSchema
from kestrapy.models.edition_provider_edition import EditionProviderEdition
from kestrapy.models.email import Email
from kestrapy.models.event_app_response import EventAppResponse
from kestrapy.models.event_execution import EventExecution
from kestrapy.models.event_execution_status_event import EventExecutionStatusEvent
from kestrapy.models.event_log_entry import EventLogEntry
from kestrapy.models.executable_task_subflow_id import ExecutableTaskSubflowId
from kestrapy.models.execution import Execution
from kestrapy.models.execution_controller_api_validate_execution_inputs_response import ExecutionControllerApiValidateExecutionInputsResponse
from kestrapy.models.execution_controller_api_validate_execution_inputs_response_api_check_failure import ExecutionControllerApiValidateExecutionInputsResponseApiCheckFailure
from kestrapy.models.execution_controller_api_validate_execution_inputs_response_api_input_and_value import ExecutionControllerApiValidateExecutionInputsResponseApiInputAndValue
from kestrapy.models.execution_controller_api_validate_execution_inputs_response_api_input_error import ExecutionControllerApiValidateExecutionInputsResponseApiInputError
from kestrapy.models.execution_controller_eval_result import ExecutionControllerEvalResult
from kestrapy.models.execution_controller_execution_response import ExecutionControllerExecutionResponse
from kestrapy.models.execution_controller_last_execution_response import ExecutionControllerLastExecutionResponse
from kestrapy.models.execution_controller_set_labels_by_ids_request import ExecutionControllerSetLabelsByIdsRequest
from kestrapy.models.execution_controller_state_request import ExecutionControllerStateRequest
from kestrapy.models.execution_controller_webhook_response import ExecutionControllerWebhookResponse
from kestrapy.models.execution_kind import ExecutionKind
from kestrapy.models.execution_metadata import ExecutionMetadata
from kestrapy.models.execution_repository_interface_child_filter import ExecutionRepositoryInterfaceChildFilter
from kestrapy.models.execution_repository_interface_flow_filter import ExecutionRepositoryInterfaceFlowFilter
from kestrapy.models.execution_status_event import ExecutionStatusEvent
from kestrapy.models.execution_trigger import ExecutionTrigger
from kestrapy.models.execution_usage import ExecutionUsage
from kestrapy.models.file_attributes import FileAttributes
from kestrapy.models.file_attributes_file_type import FileAttributesFileType
from kestrapy.models.file_metas import FileMetas
from kestrapy.models.filter import Filter
from kestrapy.models.fixtures import Fixtures
from kestrapy.models.flow import Flow
from kestrapy.models.flow_controller_task_validation_type import FlowControllerTaskValidationType
from kestrapy.models.flow_for_execution import FlowForExecution
from kestrapy.models.flow_generation_prompt import FlowGenerationPrompt
from kestrapy.models.flow_graph import FlowGraph
from kestrapy.models.flow_graph_cluster import FlowGraphCluster
from kestrapy.models.flow_graph_edge import FlowGraphEdge
from kestrapy.models.flow_id import FlowId
from kestrapy.models.flow_interface import FlowInterface
from kestrapy.models.flow_node import FlowNode
from kestrapy.models.flow_relation import FlowRelation
from kestrapy.models.flow_scope import FlowScope
from kestrapy.models.flow_topology_graph import FlowTopologyGraph
from kestrapy.models.flow_topology_graph_edge import FlowTopologyGraphEdge
from kestrapy.models.flow_usage import FlowUsage
from kestrapy.models.flow_with_source import FlowWithSource
from kestrapy.models.group_identifier import GroupIdentifier
from kestrapy.models.group_identifier_membership import GroupIdentifierMembership
from kestrapy.models.group_usage import GroupUsage
from kestrapy.models.http_parameters import HttpParameters
from kestrapy.models.iam_binding_controller_api_binding_detail import IAMBindingControllerApiBindingDetail
from kestrapy.models.iam_binding_controller_api_binding_group import IAMBindingControllerApiBindingGroup
from kestrapy.models.iam_binding_controller_api_binding_user import IAMBindingControllerApiBindingUser
from kestrapy.models.iam_binding_controller_api_create_binding_request import IAMBindingControllerApiCreateBindingRequest
from kestrapy.models.iam_binding_controller_api_role import IAMBindingControllerApiRole
from kestrapy.models.iam_group_controller_api_create_group_request import IAMGroupControllerApiCreateGroupRequest
from kestrapy.models.iam_group_controller_api_group_detail import IAMGroupControllerApiGroupDetail
from kestrapy.models.iam_group_controller_api_group_member import IAMGroupControllerApiGroupMember
from kestrapy.models.iam_group_controller_api_group_membership import IAMGroupControllerApiGroupMembership
from kestrapy.models.iam_group_controller_api_update_group_request import IAMGroupControllerApiUpdateGroupRequest
from kestrapy.models.iam_invitation_controller_api_invitation_create_request import IAMInvitationControllerApiInvitationCreateRequest
from kestrapy.models.iam_invitation_controller_api_invitation_detail import IAMInvitationControllerApiInvitationDetail
from kestrapy.models.iam_invitation_controller_api_invitation_role import IAMInvitationControllerApiInvitationRole
from kestrapy.models.iam_role_controller_api_role_create_or_update_request import IAMRoleControllerApiRoleCreateOrUpdateRequest
from kestrapy.models.iam_role_controller_api_role_create_or_update_request_permissions import IAMRoleControllerApiRoleCreateOrUpdateRequestPermissions
from kestrapy.models.iam_role_controller_api_role_detail import IAMRoleControllerApiRoleDetail
from kestrapy.models.iam_service_account_controller_api_create_service_account_request import IAMServiceAccountControllerApiCreateServiceAccountRequest
from kestrapy.models.iam_service_account_controller_api_group import IAMServiceAccountControllerApiGroup
from kestrapy.models.iam_service_account_controller_api_patch_service_account_request import IAMServiceAccountControllerApiPatchServiceAccountRequest
from kestrapy.models.iam_service_account_controller_api_service_account_detail import IAMServiceAccountControllerApiServiceAccountDetail
from kestrapy.models.iam_service_account_controller_api_service_account_request import IAMServiceAccountControllerApiServiceAccountRequest
from kestrapy.models.iam_service_account_controller_api_service_account_response import IAMServiceAccountControllerApiServiceAccountResponse
from kestrapy.models.iam_tenant_access_controller_api_authentication import IAMTenantAccessControllerApiAuthentication
from kestrapy.models.iam_tenant_access_controller_api_create_tenant_access_request import IAMTenantAccessControllerApiCreateTenantAccessRequest
from kestrapy.models.iam_tenant_access_controller_api_group import IAMTenantAccessControllerApiGroup
from kestrapy.models.iam_tenant_access_controller_api_role_assignment import IAMTenantAccessControllerApiRoleAssignment
from kestrapy.models.iam_tenant_access_controller_api_tenant_access import IAMTenantAccessControllerApiTenantAccess
from kestrapy.models.iam_tenant_access_controller_api_user_permission import IAMTenantAccessControllerApiUserPermission
from kestrapy.models.iam_tenant_access_controller_api_user_tenant_access import IAMTenantAccessControllerApiUserTenantAccess
from kestrapy.models.iam_tenant_access_controller_user_api_autocomplete import IAMTenantAccessControllerUserApiAutocomplete
from kestrapy.models.iam_user_controller_api_create_or_update_user_request import IAMUserControllerApiCreateOrUpdateUserRequest
from kestrapy.models.iam_user_controller_api_group import IAMUserControllerApiGroup
from kestrapy.models.iam_user_controller_api_patch_restricted_request import IAMUserControllerApiPatchRestrictedRequest
from kestrapy.models.iam_user_controller_api_patch_user_password_request import IAMUserControllerApiPatchUserPasswordRequest
from kestrapy.models.iam_user_controller_api_user import IAMUserControllerApiUser
from kestrapy.models.iam_user_controller_api_user_auth import IAMUserControllerApiUserAuth
from kestrapy.models.iam_user_controller_api_user_summary import IAMUserControllerApiUserSummary
from kestrapy.models.iam_user_group_controller_api_update_user_groups_request import IAMUserGroupControllerApiUpdateUserGroupsRequest
from kestrapy.models.id_with_namespace import IdWithNamespace
from kestrapy.models.identity_provider import IdentityProvider
from kestrapy.models.input_object import InputObject
from kestrapy.models.input_type import InputType
from kestrapy.models.instance_controller_api_active_service import InstanceControllerApiActiveService
from kestrapy.models.instance_controller_api_active_service_list import InstanceControllerApiActiveServiceList
from kestrapy.models.instance_controller_api_create_or_update_worker_group_request import InstanceControllerApiCreateOrUpdateWorkerGroupRequest
from kestrapy.models.instance_controller_api_plugin_artifact import InstanceControllerApiPluginArtifact
from kestrapy.models.instance_controller_api_plugin_artifact_list_plugin_artifact import InstanceControllerApiPluginArtifactListPluginArtifact
from kestrapy.models.instance_controller_api_plugin_artifact_list_plugin_resolution_result import InstanceControllerApiPluginArtifactListPluginResolutionResult
from kestrapy.models.instance_controller_api_plugin_list_request import InstanceControllerApiPluginListRequest
from kestrapy.models.instance_controller_api_plugin_version_details import InstanceControllerApiPluginVersionDetails
from kestrapy.models.instance_controller_api_plugin_version_details_api_plugin_class import InstanceControllerApiPluginVersionDetailsApiPluginClass
from kestrapy.models.instance_controller_api_plugin_version_details_api_plugin_classes import InstanceControllerApiPluginVersionDetailsApiPluginClasses
from kestrapy.models.instance_controller_api_plugin_versions import InstanceControllerApiPluginVersions
from kestrapy.models.instance_controller_api_plugin_versions_api_plugin_version_and_metadata import InstanceControllerApiPluginVersionsApiPluginVersionAndMetadata
from kestrapy.models.instance_controller_api_server_instance import InstanceControllerApiServerInstance
from kestrapy.models.instance_controller_api_service_instance import InstanceControllerApiServiceInstance
from kestrapy.models.instance_controller_api_worker_group import InstanceControllerApiWorkerGroup
from kestrapy.models.instance_controller_api_worker_group_details import InstanceControllerApiWorkerGroupDetails
from kestrapy.models.instance_controller_api_worker_group_item import InstanceControllerApiWorkerGroupItem
from kestrapy.models.instance_controller_api_worker_group_list import InstanceControllerApiWorkerGroupList
from kestrapy.models.invitation import Invitation
from kestrapy.models.invitation_invitation_status import InvitationInvitationStatus
from kestrapy.models.isolation import Isolation
from kestrapy.models.kv_controller_api_delete_bulk_request import KVControllerApiDeleteBulkRequest
from kestrapy.models.kv_controller_api_delete_bulk_response import KVControllerApiDeleteBulkResponse
from kestrapy.models.kv_controller_typed_value import KVControllerTypedValue
from kestrapy.models.kv_entry import KVEntry
from kestrapy.models.kv_type import KVType
from kestrapy.models.label import Label
from kestrapy.models.left_sidebar_configuration import LeftSidebarConfiguration
from kestrapy.models.level import Level
from kestrapy.models.listener import Listener
from kestrapy.models.log_entry import LogEntry
from kestrapy.models.map_object_object import MapObjectObject
from kestrapy.models.me_controller_api_me import MeControllerApiMe
from kestrapy.models.me_controller_api_profile import MeControllerApiProfile
from kestrapy.models.me_controller_api_tenant import MeControllerApiTenant
from kestrapy.models.me_controller_api_update_password_request import MeControllerApiUpdatePasswordRequest
from kestrapy.models.me_controller_api_user_details_request import MeControllerApiUserDetailsRequest
from kestrapy.models.meta import Meta
from kestrapy.models.metric import Metric
from kestrapy.models.metric_aggregation import MetricAggregation
from kestrapy.models.metric_aggregations import MetricAggregations
from kestrapy.models.metric_entry import MetricEntry
from kestrapy.models.metric_tag import MetricTag
from kestrapy.models.misc_controller_api_usage import MiscControllerApiUsage
from kestrapy.models.misc_controller_configuration import MiscControllerConfiguration
from kestrapy.models.misc_controller_ee_configuration import MiscControllerEEConfiguration
from kestrapy.models.misc_controller_environment import MiscControllerEnvironment
from kestrapy.models.misc_controller_license_info import MiscControllerLicenseInfo
from kestrapy.models.misc_controller_plugin_id_and_version import MiscControllerPluginIdAndVersion
from kestrapy.models.misc_controller_preview import MiscControllerPreview
from kestrapy.models.misc_controller_tenant_configuration_info import MiscControllerTenantConfigurationInfo
from kestrapy.models.model_schema import ModelSchema
from kestrapy.models.name import Name
from kestrapy.models.namespace import Namespace
from kestrapy.models.namespace_allowed_namespace import NamespaceAllowedNamespace
from kestrapy.models.namespace_allowed_trigger import NamespaceAllowedTrigger
from kestrapy.models.namespace_file_revision import NamespaceFileRevision
from kestrapy.models.namespace_light import NamespaceLight
from kestrapy.models.output import Output
from kestrapy.models.output_value import OutputValue
from kestrapy.models.page_request import PageRequest
from kestrapy.models.paged_results_api_group_summary import PagedResultsApiGroupSummary
from kestrapy.models.paged_results_api_role_summary import PagedResultsApiRoleSummary
from kestrapy.models.paged_results_apps_controller_api_app import PagedResultsAppsControllerApiApp
from kestrapy.models.paged_results_apps_controller_api_app_catalog_item import PagedResultsAppsControllerApiAppCatalogItem
from kestrapy.models.paged_results_assets_controller_api_asset import PagedResultsAssetsControllerApiAsset
from kestrapy.models.paged_results_assets_controller_api_asset_usage import PagedResultsAssetsControllerApiAssetUsage
from kestrapy.models.paged_results_audit_log_controller_api_audit_log_item import PagedResultsAuditLogControllerApiAuditLogItem
from kestrapy.models.paged_results_blueprint import PagedResultsBlueprint
from kestrapy.models.paged_results_blueprint_controller_api_blueprint_item import PagedResultsBlueprintControllerApiBlueprintItem
from kestrapy.models.paged_results_concurrency_limit import PagedResultsConcurrencyLimit
from kestrapy.models.paged_results_dashboard import PagedResultsDashboard
from kestrapy.models.paged_results_execution import PagedResultsExecution
from kestrapy.models.paged_results_flow import PagedResultsFlow
from kestrapy.models.paged_results_iam_binding_controller_api_binding_detail import PagedResultsIAMBindingControllerApiBindingDetail
from kestrapy.models.paged_results_iam_group_controller_api_group_member import PagedResultsIAMGroupControllerApiGroupMember
from kestrapy.models.paged_results_iam_invitation_controller_api_invitation_detail import PagedResultsIAMInvitationControllerApiInvitationDetail
from kestrapy.models.paged_results_iam_service_account_controller_api_service_account_detail import PagedResultsIAMServiceAccountControllerApiServiceAccountDetail
from kestrapy.models.paged_results_iam_tenant_access_controller_api_user_tenant_access import PagedResultsIAMTenantAccessControllerApiUserTenantAccess
from kestrapy.models.paged_results_iam_user_controller_api_user_summary import PagedResultsIAMUserControllerApiUserSummary
from kestrapy.models.paged_results_instance_controller_api_plugin_artifact import PagedResultsInstanceControllerApiPluginArtifact
from kestrapy.models.paged_results_instance_controller_api_service_instance import PagedResultsInstanceControllerApiServiceInstance
from kestrapy.models.paged_results_kv_entry import PagedResultsKVEntry
from kestrapy.models.paged_results_log_entry import PagedResultsLogEntry
from kestrapy.models.paged_results_map_string_object import PagedResultsMapStringObject
from kestrapy.models.paged_results_metric_entry import PagedResultsMetricEntry
from kestrapy.models.paged_results_namespace import PagedResultsNamespace
from kestrapy.models.paged_results_search_result_flow import PagedResultsSearchResultFlow
from kestrapy.models.paged_results_tenant import PagedResultsTenant
from kestrapy.models.paged_results_test_suite import PagedResultsTestSuite
from kestrapy.models.paged_results_test_suite_run_result import PagedResultsTestSuiteRunResult
from kestrapy.models.paged_results_trigger import PagedResultsTrigger
from kestrapy.models.paged_results_trigger_controller_triggers import PagedResultsTriggerControllerTriggers
from kestrapy.models.patch_operation import PatchOperation
from kestrapy.models.patch_operation_path import PatchOperationPath
from kestrapy.models.patch_operation_type import PatchOperationType
from kestrapy.models.patch_request import PatchRequest
from kestrapy.models.permission import Permission
from kestrapy.models.plugin import Plugin
from kestrapy.models.plugin_artifact import PluginArtifact
from kestrapy.models.plugin_artifact_metadata import PluginArtifactMetadata
from kestrapy.models.plugin_controller_api_plugin_versions import PluginControllerApiPluginVersions
from kestrapy.models.plugin_default import PluginDefault
from kestrapy.models.plugin_icon import PluginIcon
from kestrapy.models.plugin_plugin_element_metadata import PluginPluginElementMetadata
from kestrapy.models.plugin_schema import PluginSchema
from kestrapy.models.plugin_sub_group_plugin_category import PluginSubGroupPluginCategory
from kestrapy.models.property_assets_declaration import PropertyAssetsDeclaration
from kestrapy.models.property_boolean import PropertyBoolean
from kestrapy.models.property_double import PropertyDouble
from kestrapy.models.property_duration import PropertyDuration
from kestrapy.models.property_list_string import PropertyListString
from kestrapy.models.property_object import PropertyObject
from kestrapy.models.property_string import PropertyString
from kestrapy.models.query_filter import QueryFilter
from kestrapy.models.query_filter_field import QueryFilterField
from kestrapy.models.query_filter_op import QueryFilterOp
from kestrapy.models.rbac_service_role_assignment_role_origin import RBACServiceRoleAssignmentRoleOrigin
from kestrapy.models.relation import Relation
from kestrapy.models.relation1 import Relation1
from kestrapy.models.relation_type import RelationType
from kestrapy.models.resource_type import ResourceType
from kestrapy.models.resource_type1 import ResourceType1
from kestrapy.models.resource_type_schema_extension_configuration import ResourceTypeSchemaExtensionConfiguration
from kestrapy.models.right_sidebar_configuration import RightSidebarConfiguration
from kestrapy.models.right_sidebar_configuration_custom_link import RightSidebarConfigurationCustomLink
from kestrapy.models.role import Role
from kestrapy.models.role_usage import RoleUsage
from kestrapy.models.sla import SLA
from kestrapy.models.sla_behavior import SLABehavior
from kestrapy.models.sla_labels import SLALabels
from kestrapy.models.sla_type import SLAType
from kestrapy.models.schema_attribute import SchemaAttribute
from kestrapy.models.schema_attribute_mutability import SchemaAttributeMutability
from kestrapy.models.schema_attribute_returned import SchemaAttributeReturned
from kestrapy.models.schema_attribute_type import SchemaAttributeType
from kestrapy.models.schema_attribute_uniqueness import SchemaAttributeUniqueness
from kestrapy.models.schema_type import SchemaType
from kestrapy.models.scim_extension import ScimExtension
from kestrapy.models.scim_resource import ScimResource
from kestrapy.models.scim_resource_with_optional_id import ScimResourceWithOptionalId
from kestrapy.models.scim_user import ScimUser
from kestrapy.models.search_request import SearchRequest
from kestrapy.models.search_result_flow import SearchResultFlow
from kestrapy.models.security_integration_type import SecurityIntegrationType
from kestrapy.models.server_config import ServerConfig
from kestrapy.models.server_config_liveness import ServerConfigLiveness
from kestrapy.models.server_instance import ServerInstance
from kestrapy.models.server_instance_type import ServerInstanceType
from kestrapy.models.service_instance import ServiceInstance
from kestrapy.models.service_instance_timestamped_event import ServiceInstanceTimestampedEvent
from kestrapy.models.service_provider_configuration import ServiceProviderConfiguration
from kestrapy.models.service_provider_configuration_authentication_schema import ServiceProviderConfigurationAuthenticationSchema
from kestrapy.models.service_provider_configuration_authentication_schema_type import ServiceProviderConfigurationAuthenticationSchemaType
from kestrapy.models.service_provider_configuration_bulk_configuration import ServiceProviderConfigurationBulkConfiguration
from kestrapy.models.service_provider_configuration_filter_configuration import ServiceProviderConfigurationFilterConfiguration
from kestrapy.models.service_provider_configuration_supported_configuration import ServiceProviderConfigurationSupportedConfiguration
from kestrapy.models.service_service_state import ServiceServiceState
from kestrapy.models.service_type import ServiceType
from kestrapy.models.set_apps_catalog_logo_request import SetAppsCatalogLogoRequest
from kestrapy.models.setup_configuration import SetupConfiguration
from kestrapy.models.setup_configuration_setup_data import SetupConfigurationSetupData
from kestrapy.models.sort_order import SortOrder
from kestrapy.models.sort_request import SortRequest
from kestrapy.models.state import State
from kestrapy.models.state_history import StateHistory
from kestrapy.models.state_type import StateType
from kestrapy.models.task import Task
from kestrapy.models.task_fixture import TaskFixture
from kestrapy.models.task_for_execution import TaskForExecution
from kestrapy.models.task_run import TaskRun
from kestrapy.models.task_run_attempt import TaskRunAttempt
from kestrapy.models.tenant import Tenant
from kestrapy.models.tenant_app_catalog_config import TenantAppCatalogConfig
from kestrapy.models.tenant_controller_apps_catalog_config_request import TenantControllerAppsCatalogConfigRequest
from kestrapy.models.tenant_controller_apps_catalog_config_response import TenantControllerAppsCatalogConfigResponse
from kestrapy.models.tenant_interface import TenantInterface
from kestrapy.models.tenant_usage import TenantUsage
from kestrapy.models.test_state import TestState
from kestrapy.models.test_suite import TestSuite
from kestrapy.models.test_suite_controller_run_request import TestSuiteControllerRunRequest
from kestrapy.models.test_suite_controller_search_tests_last_result import TestSuiteControllerSearchTestsLastResult
from kestrapy.models.test_suite_controller_test_suite_api_id import TestSuiteControllerTestSuiteApiId
from kestrapy.models.test_suite_controller_test_suite_bulk_request import TestSuiteControllerTestSuiteBulkRequest
from kestrapy.models.test_suite_controller_tests_last_result_response import TestSuiteControllerTestsLastResultResponse
from kestrapy.models.test_suite_run_result import TestSuiteRunResult
from kestrapy.models.test_suite_service_run_by_query_request import TestSuiteServiceRunByQueryRequest
from kestrapy.models.test_suite_service_test_run_by_query_result import TestSuiteServiceTestRunByQueryResult
from kestrapy.models.the_labels_to_pass_to_the_execution_created import TheLabelsToPassToTheExecutionCreated
from kestrapy.models.time_window import TimeWindow
from kestrapy.models.trigger import Trigger
from kestrapy.models.trigger_context import TriggerContext
from kestrapy.models.trigger_controller_set_disabled_request import TriggerControllerSetDisabledRequest
from kestrapy.models.trigger_controller_triggers import TriggerControllerTriggers
from kestrapy.models.trigger_fixture import TriggerFixture
from kestrapy.models.type import Type
from kestrapy.models.unit_test import UnitTest
from kestrapy.models.unit_test_result import UnitTestResult
from kestrapy.models.upload_versioned_plugins_request import UploadVersionedPluginsRequest
from kestrapy.models.usage_ee import UsageEE
from kestrapy.models.user_group import UserGroup
from kestrapy.models.user_group_type import UserGroupType
from kestrapy.models.user_type import UserType
from kestrapy.models.user_usage import UserUsage
from kestrapy.models.username_password_credentials import UsernamePasswordCredentials
from kestrapy.models.validate_constraint_violation import ValidateConstraintViolation
from kestrapy.models.value_path_expression import ValuePathExpression
from kestrapy.models.worker_group import WorkerGroup
from kestrapy.models.worker_group_fallback import WorkerGroupFallback
from kestrapy.models.worker_task_restart_strategy import WorkerTaskRestartStrategy




# coding: utf-8

"""
    Kestra EE

    All API operations, except for Superadmin-only endpoints, require a tenant identifier in the HTTP path.<br/> Endpoints designated as Superadmin-only are not tenant-scoped.

    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class QueryFilterOp(str, Enum):
    """
    QueryFilterOp
    """

    """
    allowed enum values
    """
    EQUALS = 'EQUALS'
    NOT_EQUALS = 'NOT_EQUALS'
    GREATER_THAN = 'GREATER_THAN'
    LESS_THAN = 'LESS_THAN'
    GREATER_THAN_OR_EQUAL_TO = 'GREATER_THAN_OR_EQUAL_TO'
    LESS_THAN_OR_EQUAL_TO = 'LESS_THAN_OR_EQUAL_TO'
    IN = 'IN'
    NOT_IN = 'NOT_IN'
    STARTS_WITH = 'STARTS_WITH'
    ENDS_WITH = 'ENDS_WITH'
    CONTAINS = 'CONTAINS'
    REGEX = 'REGEX'
    PREFIX = 'PREFIX'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of QueryFilterOp from a JSON string"""
        return cls(json.loads(json_str))





# coding: utf-8

"""
    Kestra EE

    All API operations, except for Superadmin-only endpoints, require a tenant identifier in the HTTP path.<br/> Endpoints designated as Superadmin-only are not tenant-scoped.

    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import regex as re
import json

from pydantic import BaseModel, ConfigDict, Field, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from kestrapy.models.property_boolean import PropertyBoolean
from kestrapy.models.property_double import PropertyDouble
from kestrapy.models.property_list_string import PropertyListString
from kestrapy.models.property_object import PropertyObject
from kestrapy.models.property_string import PropertyString
from typing import Optional, Set
from typing_extensions import Self

class Assertion(BaseModel):
    """
    Assertion
    """ # noqa: E501
    value: PropertyObject
    task_id: Optional[StrictStr] = Field(default=None, alias="taskId")
    error_message: Optional[PropertyString] = Field(default=None, alias="errorMessage")
    description: Optional[PropertyString] = None
    ends_with: Optional[PropertyString] = Field(default=None, alias="endsWith")
    starts_with: Optional[PropertyString] = Field(default=None, alias="startsWith")
    contains: Optional[PropertyString] = None
    equal_to: Optional[PropertyObject] = Field(default=None, alias="equalTo")
    not_equal_to: Optional[PropertyObject] = Field(default=None, alias="notEqualTo")
    greater_than: Optional[PropertyDouble] = Field(default=None, alias="greaterThan")
    greater_than_or_equal_to: Optional[PropertyDouble] = Field(default=None, alias="greaterThanOrEqualTo")
    less_than: Optional[PropertyDouble] = Field(default=None, alias="lessThan")
    less_than_or_equal_to: Optional[PropertyDouble] = Field(default=None, alias="lessThanOrEqualTo")
    var_in: Optional[PropertyListString] = Field(default=None, alias="in")
    not_in: Optional[PropertyListString] = Field(default=None, alias="notIn")
    is_null: Optional[PropertyBoolean] = Field(default=None, alias="isNull")
    is_not_null: Optional[PropertyBoolean] = Field(default=None, alias="isNotNull")
    additional_properties: Dict[str, Any] = {}
    __properties: ClassVar[List[str]] = ["value", "taskId", "errorMessage", "description", "endsWith", "startsWith", "contains", "equalTo", "notEqualTo", "greaterThan", "greaterThanOrEqualTo", "lessThan", "lessThanOrEqualTo", "in", "notIn", "isNull", "isNotNull"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of Assertion from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * Fields in `self.additional_properties` are added to the output dict.
        """
        excluded_fields: Set[str] = set([
            "additional_properties",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of value
        if self.value:
            _dict['value'] = self.value.to_dict()
        # override the default output from pydantic by calling `to_dict()` of error_message
        if self.error_message:
            _dict['errorMessage'] = self.error_message.to_dict()
        # override the default output from pydantic by calling `to_dict()` of description
        if self.description:
            _dict['description'] = self.description.to_dict()
        # override the default output from pydantic by calling `to_dict()` of ends_with
        if self.ends_with:
            _dict['endsWith'] = self.ends_with.to_dict()
        # override the default output from pydantic by calling `to_dict()` of starts_with
        if self.starts_with:
            _dict['startsWith'] = self.starts_with.to_dict()
        # override the default output from pydantic by calling `to_dict()` of contains
        if self.contains:
            _dict['contains'] = self.contains.to_dict()
        # override the default output from pydantic by calling `to_dict()` of equal_to
        if self.equal_to:
            _dict['equalTo'] = self.equal_to.to_dict()
        # override the default output from pydantic by calling `to_dict()` of not_equal_to
        if self.not_equal_to:
            _dict['notEqualTo'] = self.not_equal_to.to_dict()
        # override the default output from pydantic by calling `to_dict()` of greater_than
        if self.greater_than:
            _dict['greaterThan'] = self.greater_than.to_dict()
        # override the default output from pydantic by calling `to_dict()` of greater_than_or_equal_to
        if self.greater_than_or_equal_to:
            _dict['greaterThanOrEqualTo'] = self.greater_than_or_equal_to.to_dict()
        # override the default output from pydantic by calling `to_dict()` of less_than
        if self.less_than:
            _dict['lessThan'] = self.less_than.to_dict()
        # override the default output from pydantic by calling `to_dict()` of less_than_or_equal_to
        if self.less_than_or_equal_to:
            _dict['lessThanOrEqualTo'] = self.less_than_or_equal_to.to_dict()
        # override the default output from pydantic by calling `to_dict()` of var_in
        if self.var_in:
            _dict['in'] = self.var_in.to_dict()
        # override the default output from pydantic by calling `to_dict()` of not_in
        if self.not_in:
            _dict['notIn'] = self.not_in.to_dict()
        # override the default output from pydantic by calling `to_dict()` of is_null
        if self.is_null:
            _dict['isNull'] = self.is_null.to_dict()
        # override the default output from pydantic by calling `to_dict()` of is_not_null
        if self.is_not_null:
            _dict['isNotNull'] = self.is_not_null.to_dict()
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Assertion from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "value": PropertyObject.from_dict(obj["value"]) if obj.get("value") is not None else None,
            "taskId": obj.get("taskId"),
            "errorMessage": PropertyString.from_dict(obj["errorMessage"]) if obj.get("errorMessage") is not None else None,
            "description": PropertyString.from_dict(obj["description"]) if obj.get("description") is not None else None,
            "endsWith": PropertyString.from_dict(obj["endsWith"]) if obj.get("endsWith") is not None else None,
            "startsWith": PropertyString.from_dict(obj["startsWith"]) if obj.get("startsWith") is not None else None,
            "contains": PropertyString.from_dict(obj["contains"]) if obj.get("contains") is not None else None,
            "equalTo": PropertyObject.from_dict(obj["equalTo"]) if obj.get("equalTo") is not None else None,
            "notEqualTo": PropertyObject.from_dict(obj["notEqualTo"]) if obj.get("notEqualTo") is not None else None,
            "greaterThan": PropertyDouble.from_dict(obj["greaterThan"]) if obj.get("greaterThan") is not None else None,
            "greaterThanOrEqualTo": PropertyDouble.from_dict(obj["greaterThanOrEqualTo"]) if obj.get("greaterThanOrEqualTo") is not None else None,
            "lessThan": PropertyDouble.from_dict(obj["lessThan"]) if obj.get("lessThan") is not None else None,
            "lessThanOrEqualTo": PropertyDouble.from_dict(obj["lessThanOrEqualTo"]) if obj.get("lessThanOrEqualTo") is not None else None,
            "in": PropertyListString.from_dict(obj["in"]) if obj.get("in") is not None else None,
            "notIn": PropertyListString.from_dict(obj["notIn"]) if obj.get("notIn") is not None else None,
            "isNull": PropertyBoolean.from_dict(obj["isNull"]) if obj.get("isNull") is not None else None,
            "isNotNull": PropertyBoolean.from_dict(obj["isNotNull"]) if obj.get("isNotNull") is not None else None
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj





# coding: utf-8

"""
    Kestra EE

    All API operations, except for Superadmin-only endpoints, require a tenant identifier in the HTTP path.<br/> Endpoints designated as Superadmin-only are not tenant-scoped.

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest
from kestrapy import (
    Configuration,
    KestraClient,
    IAMRoleControllerApiRoleCreateOrUpdateRequest,
    IAMRoleControllerApiRoleCreateOrUpdateRequestPermissions,
    ApiAutocomplete,
    ApiIds,
)


class TestRolesApi(unittest.TestCase):
    """RolesApi unit test stubs"""

    def setUp(self) -> None:
        configuration = Configuration()
        configuration.host = "http://localhost:9902"
        configuration.username = "root@root.com"
        configuration.password = "Root!1234"

        self.kestra_client = KestraClient(configuration)
        self.tenant = "main"


    def tearDown(self) -> None:
        pass

    def test_autocomplete_roles(self) -> None:
        """Test case for autocomplete_roles

        List roles for autocomplete
        """
        prefix = "test_autocomplete_roles"
        permissions = IAMRoleControllerApiRoleCreateOrUpdateRequestPermissions()
        permissions.flow = ["READ"]

        role_req = IAMRoleControllerApiRoleCreateOrUpdateRequest(
            name=prefix + "_complete_roles",
            permissions=permissions,
            description="An example role"
        )

        created = self.kestra_client.roles.create_role(
            tenant=self.tenant,
            iam_role_controller_api_role_create_or_update_request=role_req
        )

        api_autocomplete = ApiAutocomplete(q=prefix)

        results = self.kestra_client.roles.autocomplete_roles(
            tenant=self.tenant,
            api_autocomplete=api_autocomplete
        )

        assert any(getattr(r, 'id', None) == created.id or getattr(r, 'name', None) == created.name for r in results)

    def test_create_role(self) -> None:
        """Test case for create_role

        Create a role
        """
        name = "test_create_role"
        permissions = IAMRoleControllerApiRoleCreateOrUpdateRequestPermissions()
        permissions.flow = ["READ"]

        role_req = IAMRoleControllerApiRoleCreateOrUpdateRequest(
            name=name,
            permissions=permissions,
            description="An example role"
        )

        created = self.kestra_client.roles.create_role(
            tenant=self.tenant,
            iam_role_controller_api_role_create_or_update_request=role_req
        )

        assert getattr(created, 'name', None) == name


    def test_delete_role(self) -> None:
        """Test case for delete_role

        Delete a role
        """
        name = "test_delete_role"
        permissions = IAMRoleControllerApiRoleCreateOrUpdateRequestPermissions()
        permissions.flow = ["READ"]

        role_req = IAMRoleControllerApiRoleCreateOrUpdateRequest(
            name=name,
            permissions=permissions,
        )

        created = self.kestra_client.roles.create_role(
            tenant=self.tenant,
            iam_role_controller_api_role_create_or_update_request=role_req
        )

        self.kestra_client.roles.delete_role(id=created.id, tenant=self.tenant)

        with self.assertRaises(Exception):
            self.kestra_client.roles.role(id=created.id, tenant=self.tenant)

    def test_get_role(self) -> None:
        """Test case for get_role

        Retrieve a role
        """
        name = "test_get_role"
        permissions = IAMRoleControllerApiRoleCreateOrUpdateRequestPermissions()
        permissions.flow = ["READ"]

        role_req = IAMRoleControllerApiRoleCreateOrUpdateRequest(
            name=name,
            permissions=permissions,
        )

        created = self.kestra_client.roles.create_role(
            tenant=self.tenant,
            iam_role_controller_api_role_create_or_update_request=role_req
        )

        fetched = self.kestra_client.roles.role(id=created.id, tenant=self.tenant)
        assert fetched.id == created.id

    def test_list_roles_from_given_ids(self) -> None:
        """Test case for list_roles_from_given_ids

        List roles by ids
        """
        name = "test_list_roles_from_given_ids"
        permissions = IAMRoleControllerApiRoleCreateOrUpdateRequestPermissions()
        permissions.flow = ["READ"]

        role_req = IAMRoleControllerApiRoleCreateOrUpdateRequest(
            name=name,
            permissions=permissions,
        )

        created = self.kestra_client.roles.create_role(
            tenant=self.tenant,
            iam_role_controller_api_role_create_or_update_request=role_req
        )

        api_ids = ApiIds(ids=[created.id])

        fetched = self.kestra_client.roles.list_roles_from_given_ids(
            tenant=self.tenant,
            api_ids=api_ids
        )

        assert len(fetched) > 0

    def test_search_roles(self) -> None:
        """Test case for search_roles

        Search for roles
        """
        name = "test_search_roles"
        permissions = IAMRoleControllerApiRoleCreateOrUpdateRequestPermissions()
        permissions.flow = ["READ"]

        role_req = IAMRoleControllerApiRoleCreateOrUpdateRequest(
            name=name,
            permissions=permissions,
        )

        created = self.kestra_client.roles.create_role(
            tenant=self.tenant,
            iam_role_controller_api_role_create_or_update_request=role_req
        )

        results = self.kestra_client.roles.search_roles(page=1, size=10, tenant=self.tenant, q=name)

        assert any(getattr(r, 'id', None) == created.id for r in results.results)

    def test_update_role(self) -> None:
        """Test case for update_role

        Update a role
        """
        name = "test_update_role"
        permissions = IAMRoleControllerApiRoleCreateOrUpdateRequestPermissions()
        permissions.flow = ["READ"]

        role_req = IAMRoleControllerApiRoleCreateOrUpdateRequest(
            name=name,
            permissions=permissions,
            description="Before"
        )

        created = self.kestra_client.roles.create_role(
            tenant=self.tenant,
            iam_role_controller_api_role_create_or_update_request=role_req
        )

        update_desc = "Updated description"
        update_req = IAMRoleControllerApiRoleCreateOrUpdateRequest(
            name=created.name,
            permissions=permissions,
            description=update_desc
        )

        updated = self.kestra_client.roles.update_role(
            id=created.id,
            tenant=self.tenant,
            iam_role_controller_api_role_create_or_update_request=update_req
        )

        assert getattr(updated, 'description', None) == update_desc


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Kestra EE

    All API operations, except for Superadmin-only endpoints, require a tenant identifier in the HTTP path.<br/> Endpoints designated as Superadmin-only are not tenant-scoped.

    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class PluginSubGroupPluginCategory(str, Enum):
    """
    PluginSubGroupPluginCategory
    """

    """
    allowed enum values
    """
    DATABASE = 'DATABASE'
    MESSAGING = 'MESSAGING'
    SCRIPT = 'SCRIPT'
    TRANSFORMATION = 'TRANSFORMATION'
    BATCH = 'BATCH'
    ALERTING = 'ALERTING'
    CLOUD = 'CLOUD'
    STORAGE = 'STORAGE'
    OTHER = 'OTHER'
    TOOL = 'TOOL'
    AI = 'AI'
    CORE = 'CORE'
    INGESTION = 'INGESTION'
    BI = 'BI'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of PluginSubGroupPluginCategory from a JSON string"""
        return cls(json.loads(json_str))





# coding: utf-8

"""
    Kestra EE

    All API operations, except for Superadmin-only endpoints, require a tenant identifier in the HTTP path.<br/> Endpoints designated as Superadmin-only are not tenant-scoped.

    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class ConcurrencyBehavior(str, Enum):
    """
    ConcurrencyBehavior
    """

    """
    allowed enum values
    """
    QUEUE = 'QUEUE'
    CANCEL = 'CANCEL'
    FAIL = 'FAIL'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of ConcurrencyBehavior from a JSON string"""
        return cls(json.loads(json_str))





# coding: utf-8

"""
    Kestra EE

    All API operations, except for Superadmin-only endpoints, require a tenant identifier in the HTTP path.<br/> Endpoints designated as Superadmin-only are not tenant-scoped.

    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import regex as re
import json

from pydantic import BaseModel, ConfigDict, Field, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from kestrapy.models.plugin_plugin_element_metadata import PluginPluginElementMetadata
from kestrapy.models.plugin_sub_group_plugin_category import PluginSubGroupPluginCategory
from typing import Optional, Set
from typing_extensions import Self

class Plugin(BaseModel):
    """
    Plugin
    """ # noqa: E501
    name: Optional[StrictStr] = None
    title: Optional[StrictStr] = None
    description: Optional[StrictStr] = None
    license: Optional[StrictStr] = None
    long_description: Optional[StrictStr] = Field(default=None, alias="longDescription")
    group: Optional[StrictStr] = None
    version: Optional[StrictStr] = None
    manifest: Optional[Dict[str, StrictStr]] = None
    guides: Optional[List[StrictStr]] = None
    aliases: Optional[List[StrictStr]] = None
    tasks: Optional[List[PluginPluginElementMetadata]] = None
    triggers: Optional[List[PluginPluginElementMetadata]] = None
    conditions: Optional[List[PluginPluginElementMetadata]] = None
    controllers: Optional[List[PluginPluginElementMetadata]] = None
    storages: Optional[List[PluginPluginElementMetadata]] = None
    secrets: Optional[List[PluginPluginElementMetadata]] = None
    task_runners: Optional[List[PluginPluginElementMetadata]] = Field(default=None, alias="taskRunners")
    apps: Optional[List[PluginPluginElementMetadata]] = None
    app_blocks: Optional[List[PluginPluginElementMetadata]] = Field(default=None, alias="appBlocks")
    charts: Optional[List[PluginPluginElementMetadata]] = None
    data_filters: Optional[List[PluginPluginElementMetadata]] = Field(default=None, alias="dataFilters")
    data_filters_kpi: Optional[List[PluginPluginElementMetadata]] = Field(default=None, alias="dataFiltersKPI")
    log_exporters: Optional[List[PluginPluginElementMetadata]] = Field(default=None, alias="logExporters")
    additional_plugins: Optional[List[PluginPluginElementMetadata]] = Field(default=None, alias="additionalPlugins")
    categories: Optional[List[PluginSubGroupPluginCategory]] = None
    sub_group: Optional[StrictStr] = Field(default=None, alias="subGroup")
    additional_properties: Dict[str, Any] = {}
    __properties: ClassVar[List[str]] = ["name", "title", "description", "license", "longDescription", "group", "version", "manifest", "guides", "aliases", "tasks", "triggers", "conditions", "controllers", "storages", "secrets", "taskRunners", "apps", "appBlocks", "charts", "dataFilters", "dataFiltersKPI", "logExporters", "additionalPlugins", "categories", "subGroup"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of Plugin from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * Fields in `self.additional_properties` are added to the output dict.
        """
        excluded_fields: Set[str] = set([
            "additional_properties",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in tasks (list)
        _items = []
        if self.tasks:
            for _item_tasks in self.tasks:
                if _item_tasks:
                    _items.append(_item_tasks.to_dict())
            _dict['tasks'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in triggers (list)
        _items = []
        if self.triggers:
            for _item_triggers in self.triggers:
                if _item_triggers:
                    _items.append(_item_triggers.to_dict())
            _dict['triggers'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in conditions (list)
        _items = []
        if self.conditions:
            for _item_conditions in self.conditions:
                if _item_conditions:
                    _items.append(_item_conditions.to_dict())
            _dict['conditions'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in controllers (list)
        _items = []
        if self.controllers:
            for _item_controllers in self.controllers:
                if _item_controllers:
                    _items.append(_item_controllers.to_dict())
            _dict['controllers'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in storages (list)
        _items = []
        if self.storages:
            for _item_storages in self.storages:
                if _item_storages:
                    _items.append(_item_storages.to_dict())
            _dict['storages'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in secrets (list)
        _items = []
        if self.secrets:
            for _item_secrets in self.secrets:
                if _item_secrets:
                    _items.append(_item_secrets.to_dict())
            _dict['secrets'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in task_runners (list)
        _items = []
        if self.task_runners:
            for _item_task_runners in self.task_runners:
                if _item_task_runners:
                    _items.append(_item_task_runners.to_dict())
            _dict['taskRunners'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in apps (list)
        _items = []
        if self.apps:
            for _item_apps in self.apps:
                if _item_apps:
                    _items.append(_item_apps.to_dict())
            _dict['apps'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in app_blocks (list)
        _items = []
        if self.app_blocks:
            for _item_app_blocks in self.app_blocks:
                if _item_app_blocks:
                    _items.append(_item_app_blocks.to_dict())
            _dict['appBlocks'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in charts (list)
        _items = []
        if self.charts:
            for _item_charts in self.charts:
                if _item_charts:
                    _items.append(_item_charts.to_dict())
            _dict['charts'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in data_filters (list)
        _items = []
        if self.data_filters:
            for _item_data_filters in self.data_filters:
                if _item_data_filters:
                    _items.append(_item_data_filters.to_dict())
            _dict['dataFilters'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in data_filters_kpi (list)
        _items = []
        if self.data_filters_kpi:
            for _item_data_filters_kpi in self.data_filters_kpi:
                if _item_data_filters_kpi:
                    _items.append(_item_data_filters_kpi.to_dict())
            _dict['dataFiltersKPI'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in log_exporters (list)
        _items = []
        if self.log_exporters:
            for _item_log_exporters in self.log_exporters:
                if _item_log_exporters:
                    _items.append(_item_log_exporters.to_dict())
            _dict['logExporters'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in additional_plugins (list)
        _items = []
        if self.additional_plugins:
            for _item_additional_plugins in self.additional_plugins:
                if _item_additional_plugins:
                    _items.append(_item_additional_plugins.to_dict())
            _dict['additionalPlugins'] = _items
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Plugin from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "name": obj.get("name"),
            "title": obj.get("title"),
            "description": obj.get("description"),
            "license": obj.get("license"),
            "longDescription": obj.get("longDescription"),
            "group": obj.get("group"),
            "version": obj.get("version"),
            "manifest": obj.get("manifest"),
            "guides": obj.get("guides"),
            "aliases": obj.get("aliases"),
            "tasks": [PluginPluginElementMetadata.from_dict(_item) for _item in obj["tasks"]] if obj.get("tasks") is not None else None,
            "triggers": [PluginPluginElementMetadata.from_dict(_item) for _item in obj["triggers"]] if obj.get("triggers") is not None else None,
            "conditions": [PluginPluginElementMetadata.from_dict(_item) for _item in obj["conditions"]] if obj.get("conditions") is not None else None,
            "controllers": [PluginPluginElementMetadata.from_dict(_item) for _item in obj["controllers"]] if obj.get("controllers") is not None else None,
            "storages": [PluginPluginElementMetadata.from_dict(_item) for _item in obj["storages"]] if obj.get("storages") is not None else None,
            "secrets": [PluginPluginElementMetadata.from_dict(_item) for _item in obj["secrets"]] if obj.get("secrets") is not None else None,
            "taskRunners": [PluginPluginElementMetadata.from_dict(_item) for _item in obj["taskRunners"]] if obj.get("taskRunners") is not None else None,
            "apps": [PluginPluginElementMetadata.from_dict(_item) for _item in obj["apps"]] if obj.get("apps") is not None else None,
            "appBlocks": [PluginPluginElementMetadata.from_dict(_item) for _item in obj["appBlocks"]] if obj.get("appBlocks") is not None else None,
            "charts": [PluginPluginElementMetadata.from_dict(_item) for _item in obj["charts"]] if obj.get("charts") is not None else None,
            "dataFilters": [PluginPluginElementMetadata.from_dict(_item) for _item in obj["dataFilters"]] if obj.get("dataFilters") is not None else None,
            "dataFiltersKPI": [PluginPluginElementMetadata.from_dict(_item) for _item in obj["dataFiltersKPI"]] if obj.get("dataFiltersKPI") is not None else None,
            "logExporters": [PluginPluginElementMetadata.from_dict(_item) for _item in obj["logExporters"]] if obj.get("logExporters") is not None else None,
            "additionalPlugins": [PluginPluginElementMetadata.from_dict(_item) for _item in obj["additionalPlugins"]] if obj.get("additionalPlugins") is not None else None,
            "categories": obj.get("categories"),
            "subGroup": obj.get("subGroup")
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj





# coding: utf-8

"""
    Kestra EE

    All API operations, except for Superadmin-only endpoints, require a tenant identifier in the HTTP path.<br/> Endpoints designated as Superadmin-only are not tenant-scoped.

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest
from kestrapy import Configuration, KestraClient, FlowControllerTaskValidationType
from typing import Optional
import time
import uuid
from kestrapy import (
    IdWithNamespace,
    QueryFilter,
    QueryFilterField,
    QueryFilterOp,
)
from kestrapy.models.delete_executions_by_query_request import DeleteExecutionsByQueryRequest

class TestFlowsApi(unittest.TestCase):
    """FlowsApi unit test stubs"""

    def setUp(self) -> None:
        configuration = Configuration()
        configuration.host = "http://localhost:9902"
        configuration.username = "root@root.com"
        configuration.password = "Root!1234"

        self.kestra_client = KestraClient(configuration)
        self.tenant = "main"

    def tearDown(self) -> None:
        pass

    def create_flow(self, flow_id: Optional[str] = None, namespace: str = "test.flows") -> str:
        """Helper to create a simple flow from a YAML source and return its id."""
        if flow_id is None:
            # include test method name to avoid collisions
            flow_id = f"{self._testMethodName}_{uuid.uuid4().hex[:6]}"

        body = f"""id: {flow_id}
namespace: {namespace}

tasks:
  - id: hello
    type: io.kestra.plugin.core.flow.Sleep
    duration: PT1S
"""
        created = self.kestra_client.flows.create_flow(tenant=self.tenant, body=body)
        return getattr(created, 'id', flow_id)

    def test_bulk_update_flows(self) -> None:
        """Test case for bulk_update_flows

        Update from multiples yaml sources
        """
        pass

    def test_create_complete_flow(self) -> None:
        """Test case for create_complete_flow

        Create a complete flow from yaml source
        """
        body = """id: flow_complete
namespace: tests

description: Let's `write` some **markdown**

labels:
  team: data
  owner: kestrel
  project: falco
  environment: dev
  country: US

inputs:
  - id: user
    type: STRING
    required: true
    defaults: Kestrel
    description: This is an optional input — if not set at runtime, it will use the default value Kestrel

  - id: run_task
    type: BOOL
    defaults: true

  - id: pokemon
    type: MULTISELECT
    displayName: Choose your favorite Pokemon
    description: You can pick more than one!
    values:
      - Pikachu
      - Charizard
      - Bulbasaur
      - Psyduck
      - Squirtle
      - Mewtwo
      - Snorlax
    dependsOn:
      inputs:
        - run_task
      condition: "{{ inputs.run_task }}"

  - id: bird
    type: SELECT
    displayName: Choose your favorite Falco bird
    values:
      - Kestrel
      - Merlin
      - Peregrine Falcon
      - American Kestrel
    dependsOn:
      inputs:
        - user
      condition: "{{ inputs.user == 'Kestrel' }}"

variables:
  first: 1
  second: "{{ vars.first }} < 2"

tasks:
  - id: hello
    type: io.kestra.plugin.core.log.Log
    description: this is a *task* documentation
    message: |
      The variables we used are {{ vars.first }} and {{ render(vars.second) }}.
      The input is {{ inputs.user }} and the task was started at {{ taskrun.startDate }} from flow {{ flow.id }}.

  - id: run_if_true
    type: io.kestra.plugin.core.debug.Return
    format: Hello World!
    runIf: "{{ inputs.run_task }}"

  - id: fallback
    type: io.kestra.plugin.core.debug.Return
    format: fallback output

finally:
  - id: finally_log
    type: io.kestra.plugin.core.log.Log
    message: "This task runs after all the tasks are run, irrespective of whether the tasks ran successfully or failed. Execution {{ execution.state }}" # Execution RUNNING

afterExecution:
  - id: afterExecution_log
    type: io.kestra.plugin.core.log.Log
    message: "This task runs after the flow execution is complete. Execution {{ execution.state }}" # Execution FAILED / SUCCESS

outputs:
  - id: flow_output
    type: STRING
    value: "{{ tasks.run_if_true.state != 'SKIPPED' ? outputs.run_if_true.value : outputs.fallback.value }}"

pluginDefaults:
  - type: io.kestra.plugin.core.log.Log
    values:
      level: TRACE

triggers:
  - id: monthly
    type: io.kestra.plugin.core.trigger.Schedule
    cron: "0 9 1 * *" # 1st of each month at 9am
"""

        created = self.kestra_client.flows.create_flow(tenant=self.tenant, body=body)

        assert created.id == "flow_complete"

    def test_create_flow(self) -> None:
        """Test case for create_flow

        Create a flow from yaml source
        """
        flow_id = f"{self._testMethodName}_flow"
        created_id = self.create_flow(flow_id=flow_id)
        assert created_id == flow_id

    def test_delete_flow(self) -> None:
        """Test case for delete_flow

        Delete a flow
        """
        flow_id = f"{self._testMethodName}_flow"
        created_id = self.create_flow(flow_id=flow_id)

        # delete using correct parameter name 'id'
        self.kestra_client.flows.delete_flow(namespace="test.flows", id=created_id, tenant=self.tenant)

        # get_flow requires source and allow_deleted parameters
        with self.assertRaises(Exception):
            self.kestra_client.flows.flow(namespace="test.flows", id=created_id, source=False, allow_deleted=False, tenant=self.tenant)

    def test_delete_flows_by_ids(self) -> None:
        """Test case for delete_flows_by_ids

        Delete flows by their IDs.
        """
        # create two flows with ids derived from test name
        f1 = f"{self._testMethodName}_1"
        f2 = f"{self._testMethodName}_2"
        id1 = self.create_flow(flow_id=f1)
        id2 = self.create_flow(flow_id=f2)

        req = [IdWithNamespace(id=id1, namespace="test.flows"), IdWithNamespace(id=id2, namespace="test.flows")]
        resp = self.kestra_client.flows.delete_flows_by_ids(tenant=self.tenant, id_with_namespace=req)
        # expect a BulkResponse-like object or no exception
        assert resp is not None

        with self.assertRaises(Exception):
            self.kestra_client.flows.flow(namespace="test.flows", id=id1, source=False, allow_deleted=False, tenant=self.tenant)
        with self.assertRaises(Exception):
            self.kestra_client.flows.flow(namespace="test.flows", id=id2, source=False, allow_deleted=False, tenant=self.tenant)

    def test_delete_flows_by_query(self) -> None:
        """Test case for delete_flows_by_query

        Delete flows returned by the query parameters.
        """
        base = f"{self._testMethodName}"
        namespace = "test_delete_flows_by_query"
        id_a = self.create_flow(flow_id=f"{base}_a", namespace=namespace)
        id_b = self.create_flow(flow_id=f"{base}_b", namespace=namespace)

        qf = QueryFilter(field=QueryFilterField.NAMESPACE, operation=QueryFilterOp.CONTAINS, value={"value": namespace})

        resp = self.kestra_client.flows.delete_flows_by_query(tenant=self.tenant, filters=[qf])
        assert resp is not None

        with self.assertRaises(Exception):
            self.kestra_client.flows.flow(namespace=namespace, id=id_a, source=False, allow_deleted=False, tenant=self.tenant)
        with self.assertRaises(Exception):
            self.kestra_client.flows.flow(namespace=namespace, id=id_b, source=False, allow_deleted=False, tenant=self.tenant)

    def test_disable_flows_by_ids(self) -> None:
        """Test case for disable_flows_by_ids

        Disable flows by their IDs.
        """
        f1 = f"{self._testMethodName}_1"
        f2 = f"{self._testMethodName}_2"
        id1 = self.create_flow(flow_id=f1)
        id2 = self.create_flow(flow_id=f2)

        items = [IdWithNamespace(id=id1, namespace="test.flows"), IdWithNamespace(id=id2, namespace="test.flows")]
        resp = self.kestra_client.flows.disable_flows_by_ids(tenant=self.tenant, id_with_namespace=items)
        assert resp is not None

    def test_disable_flows_by_query(self) -> None:
        """Test case for disable_flows_by_query

        Disable flows returned by the query parameters.
        """
        base = f"{self._testMethodName}"
        namespace = "test_delete_flows_by_query"
        id_a = self.create_flow(flow_id=f"{base}_a", namespace=namespace)
        id_b = self.create_flow(flow_id=f"{base}_b", namespace=namespace)

        qf = QueryFilter(field=QueryFilterField.NAMESPACE, operation=QueryFilterOp.CONTAINS, value={"value": namespace})
        req = DeleteExecutionsByQueryRequest(filters=[qf])

        resp = self.kestra_client.flows.disable_flows_by_query(tenant=self.tenant, filters=[qf])
        assert resp is not None

    def test_enable_flows_by_ids(self) -> None:
        """Test case for enable_flows_by_ids

        Enable flows by their IDs.
        """
        f1 = f"{self._testMethodName}_1"
        f2 = f"{self._testMethodName}_2"
        id1 = self.create_flow(flow_id=f1)
        id2 = self.create_flow(flow_id=f2)

        items = [IdWithNamespace(id=id1, namespace="test.flows"), IdWithNamespace(id=id2, namespace="test.flows")]
        resp = self.kestra_client.flows.enable_flows_by_ids(tenant=self.tenant, id_with_namespace=items)
        assert resp is not None

    def test_enable_flows_by_query(self) -> None:
        """Test case for enable_flows_by_query

        Enable flows returned by the query parameters.
        """
        base = f"{self._testMethodName}"
        namespace = "test_delete_flows_by_query"
        id_a = self.create_flow(flow_id=f"{base}_a", namespace=namespace)
        id_b = self.create_flow(flow_id=f"{base}_b", namespace=namespace)

        qf = QueryFilter(field=QueryFilterField.NAMESPACE, operation=QueryFilterOp.CONTAINS, value={"value": namespace})
        req = DeleteExecutionsByQueryRequest(filters=[qf])

        # enable_flows_by_query expects a DeleteExecutionsByQueryRequest in this SDK
        resp = self.kestra_client.flows.enable_flows_by_query(tenant=self.tenant, filters=[qf])
        assert resp is not None

    def test_export_flows_by_ids(self) -> None:
        """Test case for export_flows_by_ids

        Export flows as a ZIP archive of yaml sources.
        """
        f1 = f"{self._testMethodName}_1"
        f2 = f"{self._testMethodName}_2"
        id1 = self.create_flow(flow_id=f1)
        id2 = self.create_flow(flow_id=f2)

        items = [IdWithNamespace(id=id1, namespace="test.flows"), IdWithNamespace(id=id2, namespace="test.flows")]
        resp = self.kestra_client.flows.export_flows_by_ids(tenant=self.tenant, id_with_namespace=items)
        assert resp is not None
        assert isinstance(resp, (bytes, bytearray))

    def test_export_flows_by_query(self) -> None:
        """Test case for export_flows_by_query

        Export flows as a ZIP archive of yaml sources.
        """
        base = f"{self._testMethodName}"
        namespace = f"{self._testMethodName}"
        id_a = self.create_flow(flow_id=f"{base}_a", namespace=namespace)
        id_b = self.create_flow(flow_id=f"{base}_b", namespace=namespace)

        qf = QueryFilter(field=QueryFilterField.QUERY, operation=QueryFilterOp.EQUALS, value={"value": base})
        resp = self.kestra_client.flows.export_flows_by_query(tenant=self.tenant, filters=[qf])
        # expect a binary zip-like response (bytearray/bytes)
        assert resp is not None
        assert isinstance(resp, (bytes, bytearray))

    def test_generate_flow_graph(self) -> None:
        """Test case for generate_flow_graph

        Generate a graph for a flow
        """
        flow_id = f"{self._testMethodName}_flow"
        created_id = self.create_flow(flow_id=flow_id)

        resp = self.kestra_client.flows.generate_flow_graph(namespace="test.flows", id=created_id, tenant=self.tenant)
        assert resp is not None

    def test_generate_flow_graph_from_source(self) -> None:
        """Test case for generate_flow_graph_from_source

        Generate a graph for a flow source
        """
        flow_id = f"{self._testMethodName}_flow"
        body = f"""id: {flow_id}
namespace: test.flows

tasks:
  - id: hello
    type: io.kestra.plugin.core.flow.Sleep
    duration: PT1S
"""
        resp = self.kestra_client.flows.generate_flow_graph_from_source(tenant=self.tenant, body=body)
        assert resp is not None

    def test_get_flow(self) -> None:
        """Test case for get_flow

        Get a flow
        """
        flow_id = f"{self._testMethodName}_flow"
        created_id = self.create_flow(flow_id=flow_id)

        fetched = self.kestra_client.flows.flow(namespace="test.flows", id=created_id, source=False, allow_deleted=False, tenant=self.tenant)
        fetched_id = getattr(fetched, 'id', None) if not isinstance(fetched, dict) else fetched.get('id')
        assert fetched_id == created_id

    def test_get_flow_dependencies(self) -> None:
        """Test case for get_flow_dependencies

        Get flow dependencies
        """
        namespace = f"{self._testMethodName}"
        flow_id = f"{self._testMethodName}_flow"
        created_id = self.create_flow(flow_id=flow_id, namespace=namespace)

        subflow_id = f"{self._testMethodName}_subflow"
        body = f"""id: {subflow_id}
namespace: {namespace}

tasks:
  - id: hello
    type: io.kestra.plugin.core.flow.Subflow
    namespace: {namespace}
    flowId: {flow_id}
"""
        subflow = self.kestra_client.flows.create_flow(tenant=self.tenant, body=body)
        time.sleep(0.1)

        deps = self.kestra_client.flows.flow_dependencies(namespace=namespace, id=created_id, destination_only=False, expand_all=False, tenant=self.tenant)
        assert deps is not None
        assert len(deps.nodes) > 0

    def test_get_flow_dependencies_from_namespace(self) -> None:
        """Test case for get_flow_dependencies_from_namespace

        Retrieve flow dependencies
        """
        namespace = f"{self._testMethodName}"
        flow_id = f"{self._testMethodName}_flow"
        self.create_flow(flow_id=flow_id, namespace=namespace)

        subflow_id = f"{self._testMethodName}_subflow"
        body = f"""id: {subflow_id}
namespace: {namespace}

tasks:
  - id: hello
    type: io.kestra.plugin.core.flow.Subflow
    namespace: {namespace}
    flowId: {flow_id}
    
"""
        self.kestra_client.flows.create_flow(tenant=self.tenant, body=body)

        time.sleep(0.1)
        deps = self.kestra_client.flows.flow_dependencies_from_namespace(namespace=namespace, destination_only=False, tenant=self.tenant)
        assert deps is not None
        assert len(deps.nodes) > 0

    def test_get_task_from_flow(self) -> None:
        """Test case for get_task_from_flow

        Get a flow task
        """
        pass

    def test_import_flows(self) -> None:
        """Test case for import_flows

            Import flows as a ZIP archive of yaml sources or a multi-objects YAML file.     When sending a Yaml that contains one or more flows, a list of index is returned.     When sending a ZIP archive, a list of files that couldn't be imported is returned. 
        """
        pass

    def test_list_distinct_namespaces(self) -> None:
        """Test case for list_distinct_namespaces

        List all distinct namespaces
        """
        pass

    def test_list_flow_revisions(self) -> None:
        """Test case for list_flow_revisions

        Get revisions for a flow
        """
        flow_id = f"{self._testMethodName}_flow"
        created_id = self.create_flow(flow_id=flow_id)

        # update the flow to create a new revision
        body_updated = f"""id: {flow_id}
namespace: test.flows

tasks:
  - id: hello
    type: io.kestra.plugin.core.log.Log
    message: "updated message"
"""
        self.kestra_client.flows.update_flow(id=created_id, namespace="test.flows", tenant=self.tenant, body=body_updated)

        revisions = self.kestra_client.flows.list_flow_revisions(namespace="test.flows", id=created_id, tenant=self.tenant)
        assert revisions is not None
        assert len(revisions) >= 1

    def test_list_flows_by_namespace(self) -> None:
        """Test case for list_flows_by_namespace

        Retrieve all flows from a given namespace
        """
        namespace = f"test.list_ns.{uuid.uuid4().hex[:6]}"
        f1 = f"{self._testMethodName}_1"
        f2 = f"{self._testMethodName}_2"
        id1 = self.create_flow(flow_id=f1, namespace=namespace)
        id2 = self.create_flow(flow_id=f2, namespace=namespace)

        flows = self.kestra_client.flows.list_flows_by_namespace(namespace=namespace, tenant=self.tenant)
        assert flows is not None
        ids = {getattr(f, 'id', None) for f in flows}
        assert id1 in ids
        assert id2 in ids

    def test_search_flows(self) -> None:
        """Test case for search_flows

        Search for flows
        """
        base = f"{self._testMethodName}_{uuid.uuid4().hex[:6]}"
        namespace = f"search_ns_{uuid.uuid4().hex[:4]}"
        created_id = self.create_flow(flow_id=base, namespace=namespace)

        qf = QueryFilter(field=QueryFilterField.QUERY, operation=QueryFilterOp.EQUALS, value={"value": base})
        results = self.kestra_client.flows.search_flows(page=1, size=10, tenant=self.tenant, filters=[qf])
        assert results is not None
        assert hasattr(results, 'results')
        assert results.total >= 1
        found_ids = {getattr(r, 'id', None) for r in results.results}
        assert created_id in found_ids

    def test_search_flows_by_source_code(self) -> None:
        """Test case for search_flows_by_source_code

        Search for flows source code
        """
        unique_text = f"unique-search-{uuid.uuid4().hex[:6]}"
        flow_id = f"{self._testMethodName}_flow"
        namespace = f"src_ns_{uuid.uuid4().hex[:4]}"
        body = f"""id: {flow_id}
namespace: {namespace}

description: {unique_text}

tasks:
  - id: hello
    type: io.kestra.plugin.core.flow.Sleep
    duration: PT1S
"""
        created = self.kestra_client.flows.create_flow(tenant=self.tenant, body=body)
        created_id = getattr(created, 'id', flow_id)

        results = self.kestra_client.flows.search_flows_by_source_code(page=1, size=10, tenant=self.tenant, q=unique_text, namespace=namespace)
        assert results is not None
        assert hasattr(results, 'results')
        assert results.total >= 1
        found_ids = {r.model.id for r in results.results if getattr(r, 'model', None) is not None}
        assert created_id in found_ids

    def test_update_flow(self) -> None:
        """Test case for update_flow

        Update a flow
        """
        flow_id = f"{self._testMethodName}_flow"
        namespace = f"update_ns_{uuid.uuid4().hex[:4]}"
        created_id = self.create_flow(flow_id=flow_id, namespace=namespace)

        body_updated = f"""id: {flow_id}
namespace: {namespace}

tasks:
  - id: hello
    type: io.kestra.plugin.core.log.Log
    message: "updated message"
"""
        resp = self.kestra_client.flows.update_flow(id=created_id, namespace=namespace, tenant=self.tenant, body=body_updated)
        assert resp is not None

        fetched = self.kestra_client.flows.flow(namespace=namespace, id=created_id, source=False, allow_deleted=False, tenant=self.tenant)

        assert any(t.type== 'io.kestra.plugin.core.log.Log' for t in fetched.tasks)

    def test_update_task(self) -> None:
        """Test case for update_task

        Update a single task on a flow
        """
        flow_id = f"{self._testMethodName}_flow"
        namespace = f"task_update_ns_{uuid.uuid4().hex[:4]}"
        created_id = self.create_flow(flow_id=flow_id, namespace=namespace)

        # update the flow to change the task implementation
        body_updated = f"""id: {flow_id}
namespace: {namespace}

tasks:
  - id: hello
    type: io.kestra.plugin.core.log.Log
    message: "task updated via flow update"
"""
        self.kestra_client.flows.update_flow(id=created_id, namespace=namespace, tenant=self.tenant, body=body_updated)

        # fetch the single task and verify the change
        task = self.kestra_client.flows.task_from_flow(namespace=namespace, id=created_id, task_id="hello", tenant=self.tenant)
        assert task is not None
        assert getattr(task, 'additional_properties', None)["message"] == 'task updated via flow update'

    def test_validate_flows(self) -> None:
        """Test case for validate_flows

        Validate a list of flows
        """
        body = f"""id: {self._testMethodName}
namespace: {self._testMethodName}

tasks:
  - id: hello
    type: io.kestra.plugin.core.log.Log
    message: "task updated via flow update"
"""
        self.kestra_client.flows.validate_flows(
            tenant=self.tenant,
            body=body
        )

    def test_validate_task(self) -> None:
        """Test case for validate_task

        Validate a task
        """
        trigger_dict = {
            "id": "hello",
            "type": "io.kestra.plugin.core.log.Log",
            "message": "hello world"
        }
        validate_response = self.kestra_client.flows.validate_task(
            section=FlowControllerTaskValidationType.TASKS,
            tenant=self.tenant,
            body=trigger_dict
        )
        assert validate_response.constraints is None
        assert validate_response.warnings is None

    def test_validate_trigger(self) -> None:
        """Test case for validate_trigger

        Validate trigger
        """
        trigger_dict = {
            "id": "schedule",
            "type": "io.kestra.plugin.core.trigger.Schedule",
            "cron": "*/15 * * * *"
        }
        validate_response = self.kestra_client.flows.validate_trigger(
            tenant=self.tenant,
            body=trigger_dict
        )
        assert validate_response.constraints is None
        assert validate_response.warnings is None


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Kestra EE

    All API operations, except for Superadmin-only endpoints, require a tenant identifier in the HTTP path.<br/> Endpoints designated as Superadmin-only are not tenant-scoped.

    The version of the OpenAPI document: 1.1.4
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from kestrapy.api.namespaces_api import NamespacesApi


class TestNamespacesApi(unittest.TestCase):
    """NamespacesApi unit test stubs"""

    def setUp(self) -> None:
        self.api = NamespacesApi()

    def tearDown(self) -> None:
        pass

    def test_autocomplete_namespaces(self) -> None:
        """Test case for autocomplete_namespaces

        List namespaces for autocomplete
        """
        pass

    def test_create_namespace(self) -> None:
        """Test case for create_namespace

        Create a namespace
        """
        pass

    def test_delete_namespace(self) -> None:
        """Test case for delete_namespace

        Delete a namespace
        """
        pass

    def test_delete_secret(self) -> None:
        """Test case for delete_secret

        Delete a secret for a namespace
        """
        pass

    def test_inherited_plugin_defaults(self) -> None:
        """Test case for inherited_plugin_defaults

        List inherited plugin defaults
        """
        pass

    def test_inherited_secrets(self) -> None:
        """Test case for inherited_secrets

        List inherited secrets
        """
        pass

    def test_inherited_variables(self) -> None:
        """Test case for inherited_variables

        List inherited variables
        """
        pass

    def test_namespace(self) -> None:
        """Test case for namespace

        Get a namespace
        """
        pass

    def test_patch_secret(self) -> None:
        """Test case for patch_secret

        Patch a secret metadata for a namespace
        """
        pass

    def test_put_secrets(self) -> None:
        """Test case for put_secrets

        Update secrets for a namespace
        """
        pass

    def test_search_namespaces(self) -> None:
        """Test case for search_namespaces

        Search for namespaces
        """
        pass

    def test_update_namespace(self) -> None:
        """Test case for update_namespace

        Update a namespace
        """
        pass


if __name__ == '__main__':
    unittest.main()
